@@ -3,15 +3,6 @@
 #ifndef EB_H
 #define EB_H 1
 
-/* the symbol DOSLIKE is used to conditionally compile those constructs
- * that are common to DOS and NT, but not typical of Unix. */
-#ifdef MSDOS
-#define DOSLIKE 1
-#endif
-#ifdef _WIN32
-#define DOSLIKE 1
-#endif
-
 /* Define _GNU_SOURCE on Linux, so we don't have an implicit declaration
  * of asprintf, but only if we are not compiling C++.
  * Turns out that when compiling C++ on LInux, _GNU_SOURCE is helpfully
@@ -33,16 +24,8 @@
 #include <stdio.h>
 #include <errno.h>
 #include <fcntl.h>
-#include <curl/curl.h>
-#ifdef DOSLIKE
-#include <io.h>
-#include <direct.h> // for _mkdir, ...
-#include <conio.h>  // for _kbhit, getch, getche
-#include <stdint.h> // for UINT32_MAX
-#include "vsprtf.h" // for WIN32 asprintf, vasprintf, ...
-#else
 #include <unistd.h>
-#endif
+#include <curl/curl.h>
 #include <pthread.h>
 
 #ifndef O_BINARY
@@ -59,7 +42,7 @@
  * Some systems will define ushort in sys/types.h, others will not.
  * Unfortunately there is no #define symbol to key on;
  * no way to conditionally compile the following statement. */
-#if defined(DOSLIKE) || defined(__ANDROID__)
+#if defined(__ANDROID__)
 typedef unsigned short ushort;
 #endif
 /* sys/types.h defines unsigned char as unchar.  I prefer uchar.
@@ -219,6 +202,7 @@ struct i_get {
 	bool pg_ok; // watch for plugins
 	bool playonly; // only player plugins
 	bool csp; // content supresses plugins
+	bool headrequest;
 	bool is_http;
 	bool cacheable;
 	bool last_curlin;
@@ -278,7 +262,7 @@ struct DBTABLE {
 extern CURL *global_http_handle;
 extern CURLSH *global_share_handle;
 extern int debugLevel;		/* 0 to 9 */
-extern bool debugClone, debugEvent, debugThrow, debugCSS, dhs;
+extern bool debugClone, debugEvent, debugThrow, debugCSS, dhs, debugLayout;
 extern bool demin; // deminimize javascript
 extern bool uvw; // trace points
 extern bool gotimers; // run javascript timers
@@ -321,7 +305,7 @@ extern bool curlAuthNegotiate;  /* try curl negotiate (SPNEGO) auth */
 extern bool listNA;		/* list nonascii chars */
 extern bool inInput;		/* reading line from standard in */
 extern bool inInitFunction; // executing the init function
-extern int fileSize;		/* when reading/writing files */
+extern long long fileSize;		/* when reading/writing files */
 extern int timer_sn; // timer sequence numbers
 extern char errorMsg[];		/* generated error message */
 extern int localAccount;	/* this is the smtp server for outgoing mail */
@@ -339,6 +323,7 @@ extern bool fetchBlobColumns, ls_reverse;
 extern uchar ls_sort, dirWrite;
 extern char lsformat[12];
 extern bool dno; // directory names only
+extern bool flow; // send mail with format=flowed
 extern bool caseInsensitive, searchStringsAll, searchWrap, ebre;
 extern bool allowRedirection;	/* from http code 301, or http refresh */
 extern bool sendReferrer;	/* in the http header */
@@ -348,7 +333,7 @@ extern bool ftpActive;
 extern bool helpMessagesOn;	/* no need to type h */
 extern bool pluginsOn;		/* plugins are active */
 extern bool showHiddenFiles;	/* during directory scan */
-extern bool showHover; // messages that appear when you hover
+extern bool showall; // show messages that are normally hidden, for various reasons
 extern bool doColors;
 extern int context;		// which session (buffer) are we in?
 extern pst linePending;
@@ -367,6 +352,8 @@ extern char *cacheDir;	/* directory for a persistent cache of http pages */
 extern int cacheSize; // in megabytes
 extern int cacheCount; // number of cache files
 
+// General link list. This is, interestingly, the same design
+// that Fabrice came up with for his quickjs project.
 struct listHead {
 	void *next;
 	void *prev;
@@ -380,14 +367,12 @@ struct listHead {
 (e) != (void*)&(l); \
 (e) = ((struct listHead *)e)->prev)
 
-/* A pointer to the text of a line, and other line attributes */
+// Point to the text of a line.
 struct lineMap {
 	pst text;
-	char ds1, ds2;		/* directory suffix */
-	bool gflag;		/* for g// */
-	char filler;
 };
 #define LMSIZE sizeof(struct lineMap)
+#define DTSIZE 2 // size of directory type
 
 /* an edbrowse frame, as when there are many frames in an html page.
  * There could be several frames in an edbrowse window or buffer, chained
@@ -401,7 +386,7 @@ struct ebFrame {
 	int gsn; // global sequence number
 	char *fileName;		// name of file or url
 	char *firstURL;		// before http redirection
-	char *hbase; /* base for href references */
+	char *hbase; // base for href references
 	const char *charset;		// charset for this page
 	bool render1; // rendered via protocol or urlmatch
 	bool render2; // rendered via suffix
@@ -411,8 +396,8 @@ struct ebFrame {
 	bool jslink; // linke to javascript
 	bool browseMode;
 	short jtmin;
-	char *dw;		/* document.write string */
-	int dw_l;		/* length of the above */
+	char *dw;		// document.write string
+	int dw_l;		// length of the above
 // document.writes go under the body.
 	struct htmlTag *htmltag, *headtag, *bodytag;
 /* The javascript context and window corresponding to this url or frame.
@@ -442,20 +427,22 @@ struct ebWindow {
  * The back command follows this link, which is 0 if you are at the top. */
 	struct ebWindow *prev;
 	int sno; // session number
-/* This is right out of the source for ed.  Current and last line numbers. */
+// dot and dollar
 	int dot, dol;
-/* remember dot and dol for the raw text, when in browse mode */
+// remember dot and dol for the raw text, when in browse mode
 	int r_dot, r_dol;
 	int f_dot; // foreward dot, in case we browse again
 	struct ebFrame f0; /* first frame */
 	struct ebFrame *jdb_frame; // if in jdb mode
 	char *referrer; // another web page that brought this one to life
-	char *baseDirName;	/* when scanning a directory */
-	char *htmltitle, *htmldesc, *htmlkey;	/* title, description, keywords */
+	char *baseDirName;	// when scanning a directory
+	char *htmltitle, *htmldesc, *htmlkey;	// title, description, keywords
 	char *saveURL;		// for the fu command
 	char *mailInfo;
 	char lhs[MAXRE], rhs[MAXRE];	/* remembered substitution strings */
 	struct lineMap *map, *r_map;
+	struct listHead lines, r_lines;
+	char *dmap; // for directory listing
 /* The labels that you set with the k command, and access via 'x.
  * Basically, that's 26 line numbers.
  * Number 0 means the label is not set.
@@ -496,7 +483,15 @@ int ehsn;
 	bool dnoMode:1;		// directory names only
 	bool undoable:1;	/* undo is possible */
 	bool sqlMode:1;		// accessing a table
-	struct DBTABLE *table;	/* if in sqlMode */
+	bool irciMode:1;		// input irc
+	bool ircoMode:1;		// output irc
+	bool ircChannels; // show channel when displaying messages
+	FILE *ircF;
+	ushort ircOther;
+	ushort ircCount;
+	char *ircNick, *ircChannel;
+	time_t ircRespond;
+	struct DBTABLE *table;	// if in sqlMode
 	time_t nextrender;
 };
 typedef struct ebWindow Window;
@@ -518,7 +513,7 @@ the latter is at most 12 on a 64-bit machine.
 If ints are larger then I don't even use this constant.
 *********************************************************************/
 
-#define MAXLINES 170000000
+#define MAXLINES 250000000
 
 /* An edit session */
 struct ebSession {
@@ -640,7 +635,8 @@ enum {
 	TAGACT_NOP, TAGACT_JS, TAGACT_H, TAGACT_SUB, TAGACT_SUP, TAGACT_OVB,
 	TAGACT_OL, TAGACT_UL, TAGACT_DL, TAGACT_TEXT, TAGACT_BQ,
 	TAGACT_BODY, TAGACT_HEAD, TAGACT_DOC, TAGACT_FRAG, TAGACT_COMMENT,
-	TAGACT_MUSIC, TAGACT_IMAGE, TAGACT_BR, TAGACT_IBR, TAGACT_P,
+	TAGACT_MUSIC, TAGACT_SOURCE,
+	TAGACT_IMAGE, TAGACT_BR, TAGACT_IBR, TAGACT_P,
 	TAGACT_BASE, TAGACT_META, TAGACT_LINK, TAGACT_PRE, TAGACT_LABEL,
 	TAGACT_TBODY, TAGACT_THEAD, TAGACT_TFOOT,
 	TAGACT_DT, TAGACT_DD, TAGACT_LI, TAGACT_TABLE, TAGACT_TR, TAGACT_TD,
@@ -678,18 +674,9 @@ extern nodeFunction traverse_callback;
 #define BAD_BASE64_DECODE 1
 #define EXTRA_CHARS_BASE64_DECODE 2
 
-#ifdef DOSLIKE
-/* windows mkdir takes only one argument */
-#define mkdir(a,b) _mkdir(a)
-// give the above we probably don't need rwx mode but here we go
-#define MODE_rw 0600
-#define MODE_rwx 0700
-#define MODE_private 0600
-#else
 #define MODE_rw 0666
 #define MODE_rwx 0777
 #define MODE_private 0600
-#endif
 
 /* function prototypes */
 #include "ebprot.h"
