@@ -16,19 +16,19 @@ void undoSpecialClear(void);
 void removeHiddenNumbers(pst p, uchar terminate);
 pst fetchLine(int n, int show);
 void displayLine(int n);
+void printDot(void);
 void initializeReadline(void);
 pst inputLine(void);
 bool cxCompare(int cx) ;
 bool cxActive(int cx, bool error);
 bool cxQuit(int cx, int action) ;
 void cxSwitch(int cx, bool interactive) ;
+void addToMap(int nlines, int destl);
 bool addTextToBuffer(const pst inbuf, int length, int destl, bool showtrail) ;
 void delText(int start, int end) ;
 bool readFileArgv(const char *filename, int fromframe, const char *orig_head);
 bool writeFile(const char *name, int mode);
 Tag *line2frame(int ln);
-bool unfoldBufferW(const struct ebWindow *w, bool cr, char **data, int *len);
-bool unfoldBuffer(int cx, bool cr, char **data, int *len);
 bool jump2anchor(const Tag *jumptag, const char *newhash);
 bool runCommand(const char *line);
 bool edbrowseCommand(const char *line, bool script) ;
@@ -72,6 +72,7 @@ void prepareForBrowse(char *h, int h_len);
 void prepareForField(char *h);
 bool breakLine(const char *line, int len, int *newlen);
 void breakLineSetup(void);
+bool balanceLine(const char *line, int mark);
 char *htmlReformat(char *buf);
 void extractEmailAddresses(char *line);
 void cutDuplicateEmails(char *tolist, char *cclist, const char *reply);
@@ -86,6 +87,7 @@ char *iso12utf(const char *t1, const char *t2, int *lenp);
 void utfHigh(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p, bool inutf8, bool out32, bool outbig);
 char *uni2utf8(unsigned int unichar);
 void utfLow(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p, int bom);
+void diagnoseAndConvert (char **rbuf, bool *isAllocated_p, int *partSize_p, const bool firstPart, const bool showMessage);
 char *force_utf8( char *buf, int buflen);
 char *base64Encode(const char *inbuf, int inlen, bool lines);
 uchar base64Bits(char c);
@@ -136,6 +138,7 @@ void runTimer(void);
 void showTimers(void);
 void domOpensWindow(const char *href, const char *name);
 int tableType(const Tag *t);
+bool all_th(const Tag *tr);
 char *render(void);
 bool itext(int d);
 struct htmlTag *line2tr(int ln);
@@ -192,6 +195,7 @@ const char *findProxyForURL(const char *url);
 const char *findAgentForURL(const char *url);
 const char *mailRedirect(const char *to, const char *from, const char *reply, const char *subj);
 int runEbFunction(const char *line);
+const char *getInputLineFromScript(void);
 struct DBTABLE *findTableDescriptor(const char *sn);
 struct DBTABLE *newTableDescriptor(const char *name);
 bool readConfigFile(void);
@@ -282,13 +286,18 @@ void debugPrint(int lev, const char *msg, ...) ;
 void setDebugFile(const char *name); //@
 void nl(void) ;
 int perl2c(char *t) ;
-unsigned pstLength(const pst s) ;
-pst clonePstring(const pst s) ;
+unsigned pstLength(const uchar *s) ;
+pst clonePstring(const uchar *s) ;
 void copyPstring(pst s, const pst t) ;
-bool fdIntoMemory(int fd, char **data, int *len) ;
-bool fileIntoMemory(const char *filename, char **data, int *len) ;
+int comparePstring(const uchar * s, const uchar * t);
+int fdIntoMemory(int fd, char **data, int *len, bool inparts);
+int fileIntoMemory(const char *filename, char **data, int *len, bool inparts);
 bool memoryOutToFile(const char *filename, const char *data, int len, int msgcreate, int msgwrite) ;
 void truncate0(const char *filename, int fh);
+long long bufferSizeW(const Window *w, bool browsing);
+long long bufferSize(int cx, bool browsing);
+bool unfoldBufferW(const struct ebWindow *w, bool cr, char **data, int *len);
+bool unfoldBuffer(int cx, bool cr, char **data, int *len);
 void caseShift(char *s, char action) ;
 void camelCase(char *s);
 char fileTypeByName(const char *name, int showlink) ;
@@ -312,8 +321,14 @@ char getLetter(const char *s) ;
 char *getFileName(int msg, const char *defname, bool isnew, bool ws);
 int shellProtectLength(const char *s);
 void shellProtect(char *t, const char *s);
-const char *nextScanFile(const char *base); //@
-bool sortedDirList(const char *dir, struct lineMap **map_p, int *count_p, int othersort, bool reverse) ; //@
+char *dirSuffixContext(int n, int cx);
+char *dirSuffix(int n);
+char *dirSuffix2(int n, const char *path);
+char *makeAbsPath(const char *f);
+const char *nextScanFile(const char *base);
+bool readDirectory(const char *filename, int endline, char cmd, struct lineMap **map_p);
+bool delFiles(int start, int end, bool withtext, char origcmd, char *cmd_p);
+bool moveFiles(int start, int end, int dest, char origcmd);
 bool envFile(const char *line, const char **expanded); //@
 char *envFileAlloc(const char *line); //@
 bool envFileDown(const char *line, const char **expanded) ; //@
@@ -375,6 +390,12 @@ const struct MIMETYPE *findMimeByFile(const char *filename);
 const struct MIMETYPE *findMimeByContent(const char *content);
 bool runPluginCommand(const struct MIMETYPE *m, const char *inurl, const char *infile, const char *indata, int inlength, char **outdata, int *outlength);
 int playBuffer(const char *line, const char *playfile);
+void ircSetFileName(Window *w);
+bool ircWrite(void);
+void ircRead(void);
+bool ircSetup(char *line);
+void ircReadlineControl(void);
+void ircReadlineRelease(void);
 
 // sourcefile=css.c
 void writeShortCache(void);
