@@ -6,10 +6,6 @@
 
 #include "eb.h"
 
-#ifdef _MSC_VER
-#include "vsprtf.h"
-#endif
-
 #define MHLINE 512		// length of a mail header line
 // headers and other information about an email
 struct MHINFO {
@@ -36,6 +32,7 @@ struct MHINFO {
 	bool pgp;
 	bool dispat;
 	uchar error64;
+	bool startAllocated;
 };
 
 static int nattach;		// number of attachments
@@ -58,6 +55,7 @@ static void freeMailInfo(struct MHINFO *w)
 	}
 	nzFree(w->tolist);
 	nzFree(w->cclist);
+	if(w->startAllocated) nzFree(w->start);
 	nzFree(w);
 }
 
@@ -98,11 +96,25 @@ static void writeAttachment(struct MHINFO *w)
 		} else {
 			cxSwitch(cx, false);
 			i_printf(MSG_SessionX, cx);
+			int length = w->end - w-> start;
+			if (!looksBinary((uchar *) w->start, length)) {
+				diagnoseAndConvert(&w->start, &w->startAllocated, &length, true, true);
+				w->end = w->start + length; // in case of realloc
+	} else if (binaryDetect & !cw->binMode) {
+		if(debugLevel >= 1)
+			i_puts(MSG_BinaryData);
+		cw->binMode = true;
+	}
 			if (!addTextToBuffer
-			    ((pst) w->start, w->end - w->start, 0, false))
+			    ((pst) w->start, length, 0, false))
 				i_printf(MSG_AttNoCopy, cx);
-			else if (w->cfn[0])
-				cf->fileName = cloneString(w->cfn);
+			else {
+				if (debugLevel >= 1)
+					printf("%d\n", length);
+				if (w->cfn[0])
+					cf->fileName = cloneString(w->cfn);
+				cw->changeMode = false;
+			}
 			cxSwitch(svcx, false);	/* back to where we were */
 		}
 	} else if (!stringEqual(atname, "x")) {
@@ -196,8 +208,7 @@ static void setLimit(const char *t)
 	}
 	imapfetch = atoi(t);
 	earliest = early;
-	if (imapfetch < 10)
-		imapfetch = 10;
+	if (imapfetch < 5) imapfetch = 5;
 	i_printf(MSG_FetchN, imapfetch);
 }
 
@@ -716,7 +727,7 @@ action:
 		preferPlain = false;
 		printf("? ");
 		fflush(stdout);
-		key = getLetter("h?qvbefdslmnp gtwWuUa/");
+		key = getLetter("h?qvbefdsmnp gtwWuUa/");
 		printf("\b\b\b");
 		fflush(stdout);
 		if (key == '?' || key == 'h') {
@@ -751,7 +762,7 @@ imap_done:
 dispmail:
 		if (key == ' ' || key == 'g' || key == 't') {
 			if(key == 't') preferPlain = true;
-/* download the email from the imap server */
+// download the email from the imap server
 			sprintf(cust_cmd, "FETCH %d BODY[]", mif->seqno);
 			curl_easy_setopt(handle, CURLOPT_CUSTOMREQUEST,
 					 cust_cmd);
@@ -810,6 +821,7 @@ You'll see this after the perform function runs.
 /* presentMail has already freed mailstring */
 			postkey = 0;
 			if(key == 'g') goto dispmail;
+			if(strchr("wua", key)) goto reaction;
 		}
 
 		if (key == 'p') {
@@ -1771,7 +1783,7 @@ void scanMail(void)
 /* Now grab the entire message */
 		unreadStats();
 		sprintf(umf_end, "%d", unreadMin);
-		if (!fileIntoMemory(umf, &mailstring, &mailstring_l))
+		if (!fileIntoMemory(umf, &mailstring, &mailstring_l, 0))
 			showErrorAbort();
 		unreadBase = unreadMin;
 
@@ -1941,9 +1953,9 @@ key_command:
 	default:
 		i_puts(MSG_NYI);
 		goto key_command;
-	}			/* switch */
+	}			// switch
 
-/* At this point we're saving the mail somewhere. */
+// At this point we're saving the mail somewhere.
 writeMail:
 	if (!isimap || isupper(key))
 		delflag = true;
@@ -2074,7 +2086,7 @@ afterinput:
 
 	if (delflag)
 		return 'd';
-	return strchr("smvbfpg", key) ? key : 'n';
+	return strchr("smvbfpguwa", key) ? key : 'n';
 }
 
 /* Here are the common keywords for mail header lines.
@@ -2220,6 +2232,7 @@ static void ctExtras(struct MHINFO *w, const char *s, const char *t)
 	const char *q, *al, *ar;
 
 	if (w->ct < CT_MULTI) {
+// look for name= or filename=
 		quote = 0;
 		for (q = s + 1; q < t; ++q) {
 			if (isalnumByte(q[-1])) continue;
@@ -2247,10 +2260,13 @@ static void ctExtras(struct MHINFO *w, const char *s, const char *t)
 			if (ar - al >= MHLINE)
 				ar = al + MHLINE - 1;
 			strncpy(w->cfn, al, ar - al);
+// filename coulde be encoded
+			char *e = w->cfn + (ar - al);
+			isoDecode(w->cfn, &e);
+			*e = 0;
 // for security reasons, don't allow an absolute path, or even relative path
 // away from the current directory or the download directory.
 // tr / _    this seems to be what other clients do.
-			char *e;
 			for(e = w->cfn; *e; ++e)
 				if(*e == '/')
 					*e = '_';
@@ -2259,6 +2275,7 @@ static void ctExtras(struct MHINFO *w, const char *s, const char *t)
 	}
 
 	if (w->ct >= CT_MULTI) {
+// look for boundary=
 		quote = 0;
 		for (q = s + 1; q < t; ++q) {
 			if (isalnumByte(q[-1])) continue;
@@ -2285,7 +2302,7 @@ static void ctExtras(struct MHINFO *w, const char *s, const char *t)
 			strncpy(w->boundary, al, ar - al);
 			break;
 		}
-	}			/* multi or alt */
+	}			// multi or alt
 }
 
 static void isoDecode(char *vl, char **vrp)
@@ -2623,9 +2640,10 @@ static struct MHINFO *headerGlean(char *start, char *end, bool top)
 		}
 
 		if (memEqualCI(s, "content-disposition:", q - s)) {
-			if (memEqualCI(vl, "attachment", 10))
+			if (memEqualCI(vl, "attachment", 10)) {
 				w->dispat = true;
-//			ctExtras(w, s, t);
+				ctExtras(w, s, t);
+			}
 			continue;
 		}
 
@@ -2701,7 +2719,7 @@ static struct MHINFO *headerGlean(char *start, char *end, bool top)
 		printf("message: %s\n", w->mid);
 		printf("boundary: %d|%s\n", w->boundlen, w->boundary);
 		printf("filename: %s\n", w->cfn);
-		printf("length %d\n", w->end - w->start);
+		printf("length %d\n", (int)(w->end - w->start));
 		printf("content %d/%d\n", w->ct, w->ce);
 	}
 
@@ -3035,10 +3053,16 @@ static void formatMail(struct MHINFO *w, bool top)
 	int ct = w->ct;
 	int j, best;
 
-	if (w->doAttach)
-		return;
-	debugPrint(5, "format headers for content %d subject %s", ct,
-		   w->subject);
+// This use to be a test for attachment, but I got text messages
+// from a friend where the text was in an attachment.
+// So now I fold it into the display if it is plain text not encoded.
+	if (w->doAttach) {
+		if(w->ct != CT_TEXT || w->ce > CE_8BIT)
+			return;
+		stringAndString(&fm, &fm_l, mailIsHtml ? "Attachment:<br>\n" : "Attachment:\n");
+	}
+
+	debugPrint(5, "format mail content %d subject %s", ct,    w->subject);
 	stringAndString(&fm, &fm_l, headerShow(w, top));
 
 	if (ct < CT_MULTI) {
@@ -3171,6 +3195,11 @@ bool setupReply(bool all)
 		return false;
 	}
 
+	if (cw->irciMode | cw->ircoMode) {
+		setError(MSG_ReIrc);
+		return false;
+	}
+
 	if (!cw->dol) {
 		setError(MSG_ReEmpty);
 		return false;
@@ -3384,7 +3413,7 @@ found:
 	rfh = open(mailReply, O_RDONLY);
 	if (rfh < 0)
 		return;
-	if (!fdIntoMemory(rfh, &buf, &buflen))
+	if (!fdIntoMemory(rfh, &buf, &buflen, 0))
 		return;
 	close(rfh);
 
