@@ -1,4 +1,5 @@
-/* format.c, Format text, establish line breaks, manage whitespace. */
+/* format.c, Format text, establish line breaks, manage whitespace,
+iso8859, utf8, utf16, utf32, base64, color codes, emojis. */
 
 #include "eb.h"
 
@@ -924,6 +925,144 @@ void breakLineSetup(void)
 	lspace = 3;
 }
 
+// Return the number of unbalanced punctuation marks.
+// This is used by the next routine.
+static void unbalanced(char c, char d, int ln, int *back_p, int *for_p)
+{
+	char *t, *open;
+	char *p = (char *)fetchLine(ln, 1);
+	bool change;
+	char qc;
+	int backward, forward;
+
+// line is allocated, so blank out anything that looks like a string.
+	for(qc = 0, t = p; *t != '\n'; ++t) {
+		if(qc) { // in a string
+			if(*t == '\\' && t[1] != '\n') t[1] = ' ';
+			if(*t == qc) qc = 0;
+			*t = ' ';
+			continue;
+		}
+		if(*t == '"' || *t == '\'') {
+			qc = *t;
+// If we don't have a closing quote then forget it.
+			const char *u = t + 1;
+			for(; *u != '\n'; ++u) {
+				if(*u == qc) break; // found closing quote
+				if(*u == '\\' && u[1] != '\n') ++u;
+			}
+			if(*u != qc) qc = 0;
+		}
+	}
+
+	change = true;
+	while (change) {
+		change = false;
+		open = 0;
+		for (t = p; *t != '\n'; ++t) {
+			if (*t == c)
+				open = t;
+			if (*t == d && open) {
+				*open = 0;
+				*t = 0;
+				change = true;
+				open = 0;
+			}
+		}
+	}
+
+	backward = forward = 0;
+	for (t = p; *t != '\n'; ++t) {
+		if (*t == c)
+			++forward;
+		if (*t == d)
+			++backward;
+	}
+
+	free(p);
+	*back_p = backward;
+	*for_p = forward;
+}
+
+// Find the line that balances the unbalanced punctuation.
+bool balanceLine(const char *line, int mark)
+{
+	char c, d;		/* open and close */
+	char selected;
+	static char openlist[] = "{([<";
+	static char closelist[] = "})]>";
+	static const char alllist[] = "{}()[]<>`'";
+	char *t;
+	int level = 0;
+	int i, direction, forward, backward;
+
+	if ((c = *line)) {
+		const int cx = stringIsNum(line + 1);
+		if (!strchr(alllist, c) || (line[1] && cx == -1)) {
+			setError(MSG_BalanceChar, alllist);
+			return false;
+		}
+		if (!cx)
+			return true;
+		if ((t = strchr(openlist, c))) {
+			d = closelist[t - openlist];
+			direction = 1;
+		} else {
+			d = c;
+			t = strchr(closelist, d);
+			c = openlist[t - closelist];
+			direction = -1;
+		}
+		if (line[1])
+			level = cx;
+		else {
+			unbalanced(c, d, mark, &backward, &forward);
+			level = direction > 0 ? forward : backward;
+			if (!level)
+				level = 1;
+		}
+	} else {
+
+// Look for anything unbalanced, probably a brace.
+		for (i = 0; i <= 2; ++i) {
+			c = openlist[i];
+			d = closelist[i];
+			unbalanced(c, d, mark, &backward, &forward);
+			if (backward && forward) {
+				setError(MSG_BalanceAmbig, c, d, c, d);
+				return false;
+			}
+			level = backward + forward;
+			if (!level)
+				continue;
+			direction = backward ? -1 : 1;
+			break;
+		}
+		if (!level) {
+			setError(MSG_BalanceNothing);
+			return false;
+		}
+	}			/* explicit character passed in, or look for one */
+
+	selected = (direction > 0 ? c : d);
+
+/* search for the balancing line */
+	i = mark;
+	while ((i += direction) > 0 && i <= cw->dol) {
+		unbalanced(c, d, i, &backward, &forward);
+		if ((direction > 0 && backward >= level) ||
+		    (direction < 0 && forward >= level)) {
+			cw->dot = i;
+			printDot();
+			return true;
+		}
+		level += (forward - backward) * direction;
+	}			/* loop over lines */
+
+	setError(MSG_Unbalanced, selected);
+	return false;
+}
+
 char *htmlReformat(char *buf)
 {
 	const char *h, *nh, *s;
@@ -933,21 +1072,20 @@ char *htmlReformat(char *buf)
 	char *new;
 	int l, tagno, extra;
 	char *fmark;		/* mark the start of a frame */
-	const int debugPhases = 0;
 
-	if(debugPhases) printf("rendered<%s>\n", buf);
+	if(debugLayout) printf("rendered<%s>\n", buf);
 	cellDelimiters(buf);
-	if(debugPhases) printf("cells<%s>\n", buf);
+	if(debugLayout) printf("cells<%s>\n", buf);
 	html_tr(buf);
-	if(debugPhases) printf("translate<%s>\n", buf);
+	if(debugLayout) printf("translate<%s>\n", buf);
 	h3div(buf);
-	if(debugPhases) printf("h3div<%s>\n", buf);
+	if(debugLayout) printf("h3div<%s>\n", buf);
 	anchorSwap(buf);
-	if(debugPhases) printf("swap<%s>\n", buf);
+	if(debugLayout) printf("swap<%s>\n", buf);
 	anchorUnframe(buf);
-	if(debugPhases) printf("unframe<%s>\n", buf);
+	if(debugLayout) printf("unframe<%s>\n", buf);
 	html_ws(buf);
-	if(debugPhases) printf("whitespace<%s>\n", buf);
+	if(debugLayout) printf("whitespace<%s>\n", buf);
 
 	longcut = lperiod = lcomma = lright = lany = 0;
 	colno = 1;
@@ -1792,6 +1930,137 @@ void utfLow(const char *inbuf, int inbuflen, char **outbuf_p, int *outbuflen_p,
 	*outbuflen_p = obuf_l - 2;
 }
 
+// Determine type of file, utf 8 or 16 or 32, or dos mode, then convert to utf8,
+// or perhaps iso8859 for as long as we choose to support that obsolete format.
+// Show messages if appropriate.
+// Convert text and display messages if appropriate.
+// Size is passed by reference so that the new size is available
+// to the calling routine.
+// rbuf is passed by reference because some of the conversion routines
+// create a new allocated array which must be passed back.
+// This is called by readFile in buffers.c, so has some
+// confusing legacy interactions with the global variable serverData.
+void diagnoseAndConvert (char **rbuf_p, bool *isAllocated_p, int *partSize_p, const bool firstPart, const bool showMessage)
+{
+	char *rbuf = *rbuf_p;
+	char *tbuf;
+	int i, j;
+	bool crlf_yes = false, crlf_no = false, dosmode = false;
+
+// undos, only if each \n has \r preceeding.
+	if (iuConvert) {
+		for (i = 0; i < *partSize_p; ++i) {
+			if (rbuf[i] != '\n') continue;
+			if (i && rbuf[i - 1] == '\r') crlf_yes = true;
+			else crlf_no = true;
+		}
+		if (crlf_yes && !crlf_no) dosmode = true;
+	} // iuConvert
+
+	if (dosmode) {
+		if ((debugLevel >= 2 || (debugLevel == 1 && showMessage))
+		&& firstPart)
+			i_puts(MSG_ConvUnix);
+		for (i = j = 0; i < *partSize_p; ++i) {
+			char c = rbuf[i];
+			if (c == '\r' && rbuf[i + 1] == '\n')
+				continue;
+			rbuf[j++] = c;
+		}
+		rbuf[j] = 0;
+		fileSize -= (*partSize_p - j);
+		*partSize_p = j;
+		serverDataLen = fileSize;
+	}
+
+	if (!iuConvert) return;
+
+/* Classify this incoming text as ascii or 8859 or utf-x */
+	bool is8859 = false, isutf8 = false;
+	int bom = 0;
+	if(firstPart) bom = byteOrderMark((uchar *) rbuf, (int)fileSize);
+	if (bom) {
+// bom implies not reading by parts, so don't worry about that any more.
+		debugPrint(3, "text type is %s%s",
+			   ((bom & 4) ? "big " : ""),
+			   ((bom & 2) ? "utf32" : "utf16"));
+		if (debugLevel >= 2 || (debugLevel == 1 && showMessage))
+			i_puts(cons_utf8 ? MSG_ConvUtf8 :        MSG_Conv8859);
+		utfLow(rbuf, *partSize_p, &tbuf, &*partSize_p, bom);
+		if(*isAllocated_p) nzFree(rbuf);
+		*isAllocated_p = true;
+		serverData = *rbuf_p = rbuf = tbuf;
+		serverDataLen = fileSize = *partSize_p;
+	} else {
+		int oldSize = *partSize_p;
+		looks_8859_utf8((uchar *) rbuf, *partSize_p,
+				&is8859, &isutf8);
+		if(firstPart)
+		debugPrint(3, "text type is %s",
+			   (isutf8 ? "utf8"
+			    : (is8859 ? "8859" : "ascii")));
+		if (cons_utf8 && is8859) {
+			if ((debugLevel >= 2 || (debugLevel == 1 && showMessage))
+			&& firstPart)
+				i_puts(MSG_ConvUtf8);
+			iso2utf((uchar *) rbuf, *partSize_p,
+				(uchar **) & tbuf, &*partSize_p);
+			if(*isAllocated_p) nzFree(rbuf);
+			*isAllocated_p = true;
+			serverData = *rbuf_p = rbuf = tbuf;
+			fileSize += (*partSize_p - oldSize);
+			serverDataLen = fileSize;
+		}
+		if (!cons_utf8 && isutf8) {
+			if ((debugLevel >= 2 || (debugLevel == 1 && showMessage))
+			&& firstPart)
+				i_puts(MSG_Conv8859);
+			utf2iso((uchar *) rbuf, *partSize_p,
+				(uchar **) & tbuf, &*partSize_p);
+			if(*isAllocated_p) nzFree(rbuf);
+			*isAllocated_p = true;
+			serverData = *rbuf_p = rbuf = tbuf;
+			fileSize += (*partSize_p - oldSize);
+			serverDataLen = fileSize;
+		}
+		if (cons_utf8 && isutf8 && firstPart) {
+// Strip off the leading bom, if any, and no we're not going to put it back.
+			if (fileSize >= 3 &&
+			    !memcmp(rbuf, "\xef\xbb\xbf", 3)) {
+				fileSize -= 3, *partSize_p -= 3;
+				memmove(rbuf, rbuf + 3, *partSize_p);
+				serverDataLen = *partSize_p;
+			}
+		}
+	}
+
+// if reading into an empty buffer, set the mode and print message
+	if (!cw->dol) {
+		if (bom & 2) {
+			cw->utf32Mode = true;
+			debugPrint(3, "setting utf32 mode");
+		}
+		if (bom & 1) {
+			cw->utf16Mode = true;
+			debugPrint(3, "setting utf16 mode");
+		}
+		if (bom & 4)
+			cw->bigMode = true;
+		if (isutf8) {
+			cw->utf8Mode = true;
+			debugPrint(3, "setting utf8 mode");
+		}
+		if (is8859) {
+			cw->iso8859Mode = true;
+			debugPrint(3, "setting 8859 mode");
+		}
+		if (dosmode) {
+			cw->dosMode = true;
+			debugPrint(3, "setting dos mode");
+		}
+	}
+}
+
 // Convert from whatever it is to utf8, for javascript and css.
 // Result parameter is the new string, or null if no conversion.
 // But, if the original string is utf8, I remove the bom.
@@ -2346,7 +2615,7 @@ void loadEmojis(void)
 		return;
 	if(ejgroup0) // should never happen
 		clearEmojis();
-	if(!fileIntoMemory(emojiFile, &ejbase, &len)) {
+	if(!fileIntoMemory(emojiFile, &ejbase, &len, 0)) {
 		showError();
 		setError(-1);
 		return;
