@@ -2,11 +2,7 @@
 
 #include "eb.h"
 
-#ifdef _MSC_VER
-#include <fcntl.h>
-#else
 #include <signal.h>
-#endif
 #include <time.h>
 
 bool curlActive;
@@ -529,9 +525,7 @@ time_t parseHeaderDate(const char *date)
 	temptm = localtime(&now);
 	if (temptm == NULL)
 		goto fail;
-#ifndef _MSC_VER
 	utcoffset = temptm->tm_gmtoff;
-#endif
 
 	if (isdigitByte(date[0]) && isdigitByte(date[1]) &&
 	    date[2] == '-' && isalphaByte(date[3])) {
@@ -776,10 +770,9 @@ static void urlSanitize(struct i_get *g, const char *post)
 	const char *url = g->url;
 
 	if (g->uriEncoded && !looksPercented(url, post)) {
+// this test is limited to characters that really really shouldn't be there.
 		debugPrint(2, "Warning, url %s doesn't look encoded", url);
-// I'm gonna trust the flag, it generally comes from running html,
-// and it's generally right, and I'm generally wrong.
-//		g->uriEncoded = false;
+		g->uriEncoded = false;
 	}
 
 	if (!g->uriEncoded) {
@@ -1082,7 +1075,7 @@ mimestream:
 
 	still_fetching = true;
 
-	if (!post_request && presentInCache(g->urlcopy)) {
+	if (!post_request && (g->headrequest || presentInCache(g->urlcopy))) {
 		head_request = true;
 		curl_easy_setopt(h, CURLOPT_NOBODY, 1l);
 	}
@@ -1245,7 +1238,7 @@ they go where they go, so this doesn't come up very often.
 		}
 
 		if (curlret != CURLE_OK) {
-			if (!head_request)
+			if (!head_request || g->headrequest)
 				goto curl_fail;
 			ebcurl_setError(curlret, g->urlcopy, 1, g->error);
 			debugPrint(3, "switch from head to get");
@@ -1321,7 +1314,7 @@ they go where they go, so this doesn't come up very often.
 				if (curlret != CURLE_OK)
 					goto curl_fail;
 
-				if (!post_request && presentInCache(g->urlcopy)) {
+				if (!post_request && (g->headrequest || presentInCache(g->urlcopy))) {
 					head_request = true;
 					curl_easy_setopt(h, CURLOPT_NOBODY, 1l);
 				}
@@ -1380,6 +1373,15 @@ they go where they go, so this doesn't come up very often.
 			}
 		} else {	/* not redirect, not 401 */
 			if (head_request) {
+				if(g->headrequest) {
+					nzFree(g->buffer);
+					g->buffer = 0;
+					g->length = 0;
+					nzFree(referrer), referrer = 0;
+					curlret = CURLE_OK;
+					transfer_status = true;
+					break;
+				}
 				if (fetchCache
 				    (g->urlcopy, g->etag, g->modtime,
 				     &cacheData, &cacheDataLen)) {
@@ -1486,6 +1488,8 @@ void *httpConnectBack1(void *ptr)
 	addToListBack(&down_jobs, job);
 	rc = httpConnect(&g);
 	job->state = (rc ? 0 : -1);
+	nzFree(g.cfn);
+	nzFree(g.referrer);
 	return NULL;
 }
 
@@ -1502,8 +1506,8 @@ void *httpConnectBack2(void *ptr)
 	g.tsn = ++tsn;
 	debugPrint(3, "jsbg thread %d", tsn);
 	rc = httpConnect(&g);
-	nzFree(g.referrer);
 	nzFree(g.cfn);
+	nzFree(g.referrer);
 	t->loadsuccess = rc;
 	t->hcode = g.code;
 	if (rc) {
@@ -2382,16 +2386,12 @@ void setHTTPLanguage(const char *lang)
 static int
 my_curl_safeSocket(void *clientp, curl_socket_t socketfd, curlsocktype purpose)
 {
-#ifdef _MSC_VER
-	return 0;
-#else // !_MSC_VER for success = fcntl(socketfd, F_SETFD, FD_CLOEXEC);
 	int success = fcntl(socketfd, F_SETFD, FD_CLOEXEC);
 	if (success == -1)
 		success = 1;
 	else
 		success = 0;
 	return success;
-#endif // _MSC_VER y/n
 }
 
 static CURL *http_curl_init(struct i_get *g)
@@ -3162,7 +3162,7 @@ int frameExpandLine(int ln, Tag *t)
 		if (!rc) {
 /* serverData was never set, or was freed do to some other error. */
 /* We just need to pop the frame and return. */
-			fileSize = -1;	/* don't print 0 */
+			fileSize = -1;	// don't print 0
 			nzFree(cf->fileName);
 			free(cf);
 			last_f->next = 0;
@@ -3311,7 +3311,7 @@ bool reexpandFrame(void)
 	rc = readFileArgv(cf->fileName, 2, 0);
 	if (!rc) {
 /* serverData was never set, or was freed do to some other error. */
-		fileSize = -1;	/* don't print 0 */
+		fileSize = -1;	// don't print 0
 		cf = save_cf;
 		return false;
 	}
