@@ -1,25 +1,27 @@
 /*********************************************************************
-This file contains support routines for internet access, primarily http
-and https. The first batch of functions involves url parsing,
+This file contains support routines for internet access, primarily http.
+· The first batch of functions involves url parsing,
 which seems simple enough, but is actually complicated and tedious.
 All the various url encoding and decoding, and separating the url into parts.
 This is used by any internet fetch, ftp, gopher, you name it.
-After that is cookie support, for http and https.
+· Then comes cookie support, for http and https.
 Most websites require cookies to run properly.
 Set jar = path in your .ebrc for the cookie jar.
-Then there are caching functions for web pages.
+· Then there are caching functions for web pages.
 If a web page is fetched often, or recently, it will be in your cache.
 This is suppose to improve performance.
-If you want it, you should set cachedir and cachesize in your .ebrc.
-Then there are functions for http authentication.
+Set cachedir and cachesize in your .ebrc for location and size of cache.
+Set cachesize = 0 if you don't want a cache.
+· Then there are functions for http authentication.
 This is a login password mechanism that is specified in the http protocol.
 It isn't fields in a form.
 I don't think it is used very often, but sometimes it is,
 and we should support it, at least at a basic level.
-Finally, there is support for plugins, programs to run based on
+· Then support for plugins, programs to run based on
 protocol, content-type, or suffix.
 Add a plugin descriptor to your config file for each plugin you wish to support.
 The edbrowse wiki has many examples.
+· Then support for edbrowse as an irc client.
 *********************************************************************/
 
 #include "eb.h"
@@ -56,6 +58,7 @@ struct PROTOCOL {
 	{"tn3270", 0, false, false, false},
 	{"data", 0, true, false, false},
 	{"javascript", 0, true, false, false},
+	{"tel", 0, true, false, false},
 	{"git", 0, false, true, false},
 	{"svn", 0, false, true, false},
 	{"gopher", 70, false, true, true},
@@ -89,7 +92,7 @@ void unpercentURL(char *url)
 			u += 2;
 		}
 		if (!c)
-			c = ' ';	/* should never happen */
+			c = ' ';	// should never happen
 		*w++ = c;
 		if (strchr("?#\1", c))
 			break;
@@ -104,7 +107,7 @@ void unpercentURL(char *url)
 	strmove(w, u);
 }
 
-/* Unpercent an entire string. */
+// Unpercent an entire string
 void unpercentString(char *s)
 {
 	char c, *u, *w;
@@ -118,27 +121,26 @@ void unpercentString(char *s)
 			u += 2;
 		}
 		if (!c)
-			c = ' ';	/* should never happen */
+			c = ' ';	// should never happen
 		*w++ = c;
 	}
 	*w = 0;
 }
 
-/*
- * Function: percentURL
- * Arguments:
- ** start: pointer to start of input string
-  ** end: pointer to end of input string.
- * Return value: A new string with the url encoded.
- * There is an extra byte, room for / at the end.
- * This function copies its input to a dynamically-allocated buffer,
- * while performing the following transformation.  Change backslash to slash,
- * and percent-escape some of the reserved characters as per RFC3986.
- * Some of the chars retain their reserved semantics and should not be changed.
- * This is a friggin guess!
- * All characters in the area between start and end, not including end,
- * are copied or transformed, except the hash, which is removed.
- * This function is used to sanitize user-supplied URLs.  */
+/*********************************************************************
+Function: percentURL
+start and end delimit the input string.
+Return value: A new string with the url encoded.
+There is an extra byte, room for / at the end.
+This function copies its input to a dynamically-allocated buffer,
+while performing the following transformation.  Change backslash to slash,
+and percent-escape some of the reserved characters as per RFC3986.
+Some of the chars retain their reserved semantics and should not be changed.
+This is a friggin guess!
+All characters in the area between start and end, not including end,
+are copied or transformed, except the hash, which is removed.
+This function is used to sanitize user-supplied URLs. 
+*********************************************************************/
 
 /* these punctuations are percentable, anywhere in a url.
  * Google has commas in encoded URLs, and wikipedia has parentheses,
@@ -163,7 +165,7 @@ char *percentURL(const char *start, const char *end)
 	out_pointer = NULL;
 
 	for (in_pointer = start; in_pointer < end; in_pointer++)
-		if (*in_pointer <= ' ' || strchr(percentable, *in_pointer))
+		if (*(signed char *)in_pointer <= ' ' || strchr(percentable, *in_pointer))
 			bytes_to_alloc += (ESCAPED_CHAR_LENGTH - 1);
 
 	new_copy = allocMem(bytes_to_alloc);
@@ -171,7 +173,7 @@ char *percentURL(const char *start, const char *end)
 	for (in_pointer = start; in_pointer < end; in_pointer++) {
 		if (*in_pointer == '\\')
 			*out_pointer++ = '/';
-		else if (*in_pointer <= ' ' || strchr(percentable, *in_pointer)) {
+		else if (*(signed char *)in_pointer <= ' ' || strchr(percentable, *in_pointer)) {
 			*out_pointer++ = '%';
 			*out_pointer++ =
 			    hexdigits[(uchar) (*in_pointer & 0xf0) >> 4];
@@ -195,7 +197,7 @@ bool looksPercented(const char *start, const char *end)
 	if (!end)
 		end = start + strlen(start);
 	for (s = start; s < end; ++s)
-		if (*s < ' ' || strchr(percentable, *s))
+		if (*(signed char *)s <= ' ' || strchr(percentable, *s))
 			return false;
 	return true;
 }
@@ -1656,7 +1658,7 @@ void cookiesFromJar(void)
 
 	if (!cookieFile)
 		return;
-	if (!fileIntoMemory(cookieFile, &cbuf, &n))
+	if (!fileIntoMemory(cookieFile, &cbuf, &n, 0))
 		showErrorAbort();
 	cbuf[n] = 0;
 	cbuf_end = cbuf + n;
@@ -1963,11 +1965,7 @@ We don't even query the cache if we don't have at least one of etag or mod time.
 
 #define CACHECONTROLVERSION 1
 
-#ifdef DOSLIKE
-#define USLEEP(a) Sleep(a / 1000)	// sleep millisecs
-#else
 #define USLEEP(a) usleep(a)	// sleep microsecs
-#endif
 
 static int control_fh = -1;	/* file handle for cacheControl */
 static char *cache_data;
@@ -1997,23 +1995,6 @@ void setupEdbrowseCache(void)
 		close(control_fh);
 		control_fh = -1;
 	}
-#ifdef DOSLIKE
-	if (!cacheDir) {
-		if (!ebUserDir)
-			return;
-		cacheDir = allocMem(strlen(ebUserDir) + 7);
-		sprintf(cacheDir, "%s/cache", ebUserDir);
-	}
-	if (fileTypeByName(cacheDir, 0) != 'd') {
-		if (mkdir(cacheDir, 0700)) {
-/* Don't want to abort here; we might be on a readonly filesystem.
- * Don't have a cache directory and can't creat one; yet we should move on. */
-			free(cacheDir);
-			cacheDir = 0;
-			return;
-		}
-	}
-#else
 	if (!cacheDir) {
 		cacheDir = allocMem(strlen(home) + 10);
 		sprintf(cacheDir, "%s/.ebcache", home);
@@ -2027,7 +2008,6 @@ void setupEdbrowseCache(void)
 			return;
 		}
 	}
-#endif
 
 /* the cache control file, which urls go to which files, and when fetched? */
 	nzFree(cacheControl);
@@ -2071,7 +2051,7 @@ static bool readControl(void)
 	int ln = 1;
 
 	lseek(control_fh, 0L, 0);
-	if (!fdIntoMemory(control_fh, &data, &datalen))
+	if (!fdIntoMemory(control_fh, &data, &datalen, 0))
 		return false;
 
 	numentries = 0;
@@ -2319,7 +2299,7 @@ nomatch:
 match:
 	sprintf(cacheFile, "%s/%05d", cacheDir, e->filenumber);
 	if (data_len) {
-		if (!fileIntoMemory(cacheFile, data, data_len))
+		if (!fileIntoMemory(cacheFile, data, data_len, 0))
 			goto nomatch;
 	} else {
 		char *a = allocMem(strlen(cacheFile) + 5 + 1);
@@ -2557,7 +2537,7 @@ bool getUserPass(const char *url, char *creds, bool find_proxy)
 				if (memcmp(a->directory, dir, d1len))
 					continue;
 				found = a;
-			} else	/* not proxy */
+			} else	// not proxy
 				found = a;
 		}
 	}
@@ -2622,7 +2602,7 @@ addWebAuthorization(const char *url,
 		dl = dirend - dir;
 	}
 
-/* See if we've done this one before. */
+// See if we've done this one before
 	foreach(a, authlist) {
 		if (a->proxy == proxy &&
 		    a->port == port &&
@@ -3068,7 +3048,6 @@ There's no popen on windows, so here is a unix only
 fragment to use popen, which can be more efficient.
 *********************************************************************/
 
-#ifndef DOSLIKE
 	if (m->outtype && !has_o) {
 		FILE *p;
 		bool rc;
@@ -3078,13 +3057,12 @@ fragment to use popen, which can be more efficient.
 			setError(MSG_NoSpawn, cmd, errno);
 			goto fail;
 		}
-		rc = fdIntoMemory(fileno(p), outdata, outlength);
+		rc = fdIntoMemory(fileno(p), outdata, outlength, 0);
 		pclose(p);
 		if (!rc)
 			goto fail;
 		goto success;
 	}
-#endif
 
 	if (m->outtype && !has_o) {
 		strcat(cmd, " > ");
@@ -3099,7 +3077,7 @@ fragment to use popen, which can be more efficient.
 
 	if (!outdata)		// not capturing output
 		goto success;
-	if (!fileIntoMemory(outfile, outdata, outlength))
+	if (!fileIntoMemory(outfile, outdata, outlength, 0))
 		goto fail;
 // fall through
 
@@ -3166,6 +3144,10 @@ int playBuffer(const char *line, const char *playfile)
 		setError(MSG_AudioDB);
 		return 0;
 	}
+	if (cw->irciMode | cw->ircoMode) {
+		setError(MSG_AudioIrc);
+		return 0;
+	}
 	if (cw->dirMode) {
 		setError(MSG_AudioDir);
 		return 0;
@@ -3220,3 +3202,671 @@ int playBuffer(const char *line, const char *playfile)
 		rc = runPluginCommand(mt, 0, cf->fileName, 0, 0, 0, 0);
 	return rc;
 }
+
+/*********************************************************************
+native irc. Code taken from https://dl.suckless.org/tools/sic-1.2.tar.gz
+under the MIT license.
+*********************************************************************/
+
+static char irc_in[4096];
+static char irc_out[4096];
+
+static void ircSend(FILE *f, char *fmt, ...)
+{
+	va_list ap;
+	va_start(ap, fmt);
+	vsnprintf(irc_out, sizeof irc_out, fmt, ap);
+	va_end(ap);
+// he doesn't check that fprintf works, that the socket is still there,
+// I guess I won't check either, a broken ssocket will be obvious
+// the next time we try to read.
+// Note that crlf is the standard.
+	fprintf(f, "%s\r\n", irc_out);
+}
+
+// skip ahead to c
+static char*ircSkip(char *s, char c)
+{
+	while(*s != c && *s != '\0')
+		s++;
+	if(*s != '\0')
+		*s++ = '\0';
+	return s;
+}
+
+static char * ircEat(char *s, int (*p)(int), int r)
+{
+// this use to be s, I think it's a bug, I changed it to *s
+	while(*s != '\0' && p(*s) == r)
+		s++;
+	return s;
+}
+
+// This uses addTextToBuffer to place the line, and so,
+// cw has to be set to the receiving window.
+static void ircAddLine(const char *channel, bool show, const char *fmt, ...)
+{
+	va_list ap;
+	va_start(ap, fmt);
+	vsnprintf(irc_out, sizeof irc_out - 1, fmt, ap);
+	va_end(ap);
+	strcat(irc_out, "\n");
+// If leading > I will assume it's a system message,
+// and damn annoying to see the system name before each one of these.
+// If you happen to send out a message that starts with >, well then, false positive.
+	if(channel && show && irc_out[0] != '>') {
+// only if there's room, which there should be!
+		unsigned l1 = strlen(channel), l2 = strlen(irc_out);
+		if(l1 + l2 < sizeof irc_out) {
+// This is quick and dirty, just paste them together.
+// We may do something more nuanced later.
+			strmove(irc_out + l1, irc_out);
+			memcpy(irc_out, channel, l1);
+		}
+	}
+	addTextToBuffer((uchar*)irc_out, strlen(irc_out), cw->dol, false);
+}
+
+static void ircPrepLine(Window *win, Window *wout, char *line)
+{
+	Window *save_cw;
+	FILE *f = win->ircF;
+	char *usr, *par, *txt;
+	usr = win->f0.hbase;
+	if(!line || !*line)
+		return;
+	if(line[0] == ':') {
+		usr = line + 1;
+		line = ircSkip(usr, ' ');
+		if(line[0] == '\0')
+			return;
+		ircSkip(usr, '!');
+	}
+// lines come in from irc ending in crlf
+	ircSkip(line, '\r');
+	par = ircSkip(line, ' ');
+	txt = ircSkip(par, ':');
+	trimWhite(par);
+	if(stringEqual("PONG", line)) {
+		debugPrint(4, "pong back");
+		return;
+	}
+	if(!wout) return; // should never happen
+// going to call ircAddLine below
+	save_cw = cw, cw = wout;
+	if(stringEqual("PRIVMSG", line))
+		ircAddLine(par, win->ircChannels, "<%s> %s", usr, txt);
+	else if(stringEqual("PING", line)) {
+		ircSend(f, "PONG %s", txt);
+		debugPrint(4, "PING PONG %s", txt);
+	}
+	else {
+		ircAddLine(usr, win->ircChannels, ">< %s (%s): %s", line, par, txt);
+		if(stringEqual("NICK", line) && stringEqual(usr, win->ircNick)) {
+			nzFree(win->ircNick);
+			win->ircNick = cloneString(txt);
+		}
+	}
+	cw = save_cw;
+}
+
+// cw is input window
+static void ircMessage(Window *wout, const char *receiver, const char *msg)
+{
+	Window *win = cw;
+	FILE *f = win->ircF;
+	if(!receiver) {
+		debugPrint(1, "No receiver to send to");
+		return;
+	}
+	cw = wout;
+	ircAddLine(receiver, win->ircChannels, "<%s> %s",
+	(win->ircNick ? win->ircNick : emptyString), msg);
+	cw = win;
+	ircSend(f, "PRIVMSG %s :%s", receiver, msg);
+}
+
+// Set the channel on the input side, but it affects the file name on the output side
+// parameter is input window
+static void ircSetChannel(Window *w, const char *channel)
+{
+	Window *w2;
+	char *p;
+	const char *channel0 = (channel ? channel : "irc");
+	nzFree(w->ircChannel), w->ircChannel = cloneString(channel);
+	nzFree(w->f0.fileName);
+	p = allocMem(strlen(channel0) + 6);
+	sprintf(p, "%s send", channel0);
+	w->f0.fileName = p;
+	w2 = sessionList[w->ircOther].lw;
+	if(w2) ircSetFileName(w2);
+}
+
+// parameter is output window
+void ircSetFileName(Window *w)
+{
+	int i, len;
+	char *p;
+	Window *w2;
+	if(!w->ircoMode) return;
+	for(i = 1, len = 0; i < MAXSESSION; ++i) {
+		w2 = sessionList[i].lw;
+		if(!w2 || !w2->irciMode || w2->ircOther != w->sno) continue;
+		if(w2->ircChannel) len += strlen(w2->ircChannel) + 1;
+	}
+	nzFree(w->f0.fileName);
+	if(!len) { w->f0.fileName = cloneString("irc receive"); return; }
+	p = allocMem(len + 8);
+	for(i = 1, *p = 0; i < MAXSESSION; ++i) {
+		w2 = sessionList[i].lw;
+		if(!w2 || !w2->irciMode || w2->ircOther != w->sno) continue;
+		if(w2->ircChannel) {
+			strcat(p, w2->ircChannel);
+			strcat(p, " ");
+		}
+	}
+	strcat(p, "receive");
+	w->f0.fileName = p;
+}
+
+// win is the same as cw
+static void ircPrepSend(Window *win, Window *wout, char *s)
+{
+	char c, *p;
+	FILE *f = win->ircF;
+	if(s[0] == '\0')
+		return;
+	ircSkip(s, '\n');
+	if(s[0] != ':') {
+		ircMessage(wout, win->ircChannel, s);
+		return;
+	}
+	c = *++s;
+	if(c != '\0' && isspace(s[1])) {
+		p = s + 2;
+		switch(c) {
+		case 'j':
+			ircSend(f, "JOIN %s", p);
+// I'm just assuming it works
+				ircSetChannel(win, p);
+			return;
+		case 'l':
+			s = ircEat(p, isspace, 1);
+			p = ircEat(s, isspace, 0);
+			if(!*s)
+				s = (win->ircChannel ? win->ircChannel : emptyString);
+			if(*p)
+				*p++ = '\0';
+			if(!*p)
+				p = "edbrowse irc";
+			ircSend(f, "PART %s :%s", s, p);
+			if(stringEqual(s, win->ircChannel)) {
+// leaving the channel we are currently sending on,
+// what are we suppose to do?
+				ircSetChannel(win, 0);
+			}
+			return;
+		case 'm':
+			s = ircEat(p, isspace, 1);
+			p = ircEat(s, isspace, 0);
+			if(*p)
+				*p++ = '\0';
+			ircMessage(wout, s, p);
+			return;
+		case 's':
+			ircSetChannel(win, p);
+			return;
+		}
+	}
+	ircSend(f, "%s", s);
+}
+
+bool ircWrite(void)
+{
+	Window *nw = sessionList[cw->ircOther].lw;
+	int i;
+	pst s;
+	if(nw && !nw->ircoMode) nw = 0;
+	if(!nw) { // should never happen
+		setError(MSG_TextRec, cw->ircOther);
+		return false;
+	}
+	fileSize = 0;
+	for(i = 1; i <= cw->dol; ++i) {
+		s = fetchLine(i, 1);
+		fileSize += pstLength(s);
+		if(*s != '\n') { // cull empty lines
+//	*** Message to #edbrowse throttled due to flooding
+// Hopefully one second between each line is enough.
+			if(i > 1) sleep(1);
+			ircPrepSend(cw, nw, (char*)s);
+		}
+		free(s);
+	}
+	return true;
+}
+
+// on the input side
+static time_t ircNow;
+static void ircRead0(Window *w)
+{
+	Window *w2;
+	const char *emsg;
+	FILE *f = w->ircF;
+	int fd = fileno(f);
+	int rc;
+
+// find output window; should always be there
+	 w2 = sessionList[w->ircOther].lw;
+	if(w2 && !w2->ircoMode) w2 = 0;
+
+// use select to see if data is available
+	fd_set rd;
+	struct timeval tv;
+top:
+	FD_ZERO(&rd);
+	FD_SET(fd, &rd);
+	tv.tv_sec = 0; // nonblocking
+	tv.tv_usec = 0;
+	rc = select(fd + 1, &rd, 0, 0, &tv);
+	if(rc < 0) {
+// did somebody hit ^c and precisely the wrong time?
+		if(errno == EINTR)
+			return;
+// some other inexplicable error
+		emsg = " irc select error";
+		goto teardown;
+	}
+	if(rc == 0) {
+// sic would ping the host if nothing comes in for 2 minutes; we don't do that.
+// They ping us that's good enough.
+// This is where we would check ircNow against w->ircRespond + 120
+// and ping the host.
+		return;
+	}
+	if(FD_ISSET(fd, &rd)) {
+// this should always happen.
+		w->ircRespond = ircNow;
+		if(fgets(irc_in, sizeof irc_in, f) == NULL) {
+			emsg = " irc connection lost";
+			goto teardown;
+		}
+		ircPrepLine(w, w2, irc_in);
+// is there more data to get?
+		goto top;
+	}
+	return;
+
+teardown:
+	debugPrint(1, "%s%s", (w->ircChannel ? w->ircChannel : "?"), emsg);
+	fclose(f), w->ircF = 0;
+	w->ircOther = 0;
+	w->irciMode = false;
+	nzFree(w->ircNick), w->ircNick = 0;
+	nzFree(w->f0.fileName), w->f0.fileName = 0;
+	nzFree(w->f0.hbase), w->f0.hbase = 0;
+	if(w2) {
+		Window *save_cw = cw;
+		cw = w2;
+		ircAddLine(w->ircChannel, true, emsg);
+		cw = save_cw;
+		if(--w2->ircCount == 0) {
+			w2->ircoMode = false;
+			nzFree(w2->f0.fileName), w2->f0.fileName = 0;
+		}
+	}
+	nzFree(w->ircChannel), w->ircChannel = 0;
+}
+
+void ircRead(void)
+{
+	int i;
+	time(&ircNow);
+	for(i = 1; i < MAXSESSION; ++i) {
+		Window *lw = sessionList[i].lw;
+		if(lw && lw->irciMode)
+			ircRead0(lw);
+	}
+}
+
+// And now for the socket stuff, which I pretty much copied without understanding.
+
+#include <netdb.h>
+#include <sys/socket.h>
+#include <netinet/in.h>
+       #include <arpa/inet.h>
+
+typedef unsigned int IP32bit;
+#define NULL_IP (IP32bit)(-1)
+
+// Some of this tcp code resurrected from c18cf432e2ca8dca97c80aae59c710f3230cc434
+
+static int tcp_isDots(const char *s)
+{
+	const char *t;
+	char c;
+	int nd = 0;		// number of dots
+	if (!s)
+		return 0;
+	for (t = s; (c = *t); ++t) {
+		if (c == '.') {
+			++nd;
+			if (t == s || !t[1])
+				return 0;
+			if (t[-1] == '.' || t[1] == '.')
+				return 0;
+			continue;
+		}
+		if (!isdigit(c))
+			return 0;
+	}
+	return (nd == 3);
+}
+
+static IP32bit tcp_name_ip(const char *name)
+{
+	struct hostent *hp;
+	IP32bit *ip;
+	if(!name) return NULL_IP;
+	hp = gethostbyname(name);
+	if (!hp)
+#if 0
+		printf("%s\n", hstrerror(h_errno));
+#endif
+		return NULL_IP;
+#if 0
+	puts("found it");
+	if (hp->h_aliases) {
+		puts("aliases");
+		char **a = hp->h_aliases;
+		while (*a) {
+			printf("alias %s\n", *a);
+			++a;
+		}
+	}
+#endif
+	ip = (IP32bit *) * (hp->h_addr_list);
+		return !ip ? NULL_IP : *ip;
+}
+
+static char *tcp_ip_dots(IP32bit ip)
+{
+	if(ip == NULL_IP) return 0;
+	return inet_ntoa(*(struct in_addr *)&ip);
+}
+
+static char *tcp_name_dots(const char *name)
+{
+	IP32bit ip = tcp_name_ip(name);
+	if (ip == NULL_IP)
+		return 0;
+	return tcp_ip_dots(ip);
+}
+
+static IP32bit tcp_dots_ip(const char *s)
+{
+	struct in_addr a;
+	if(!s) return NULL_IP;
+// Why can't SCO Unix be like everybody else?
+#ifdef SCO
+	inet_aton(s, &a);
+#else
+	*(IP32bit *) & a = inet_addr(s);
+#endif
+	return *(IP32bit *) & a;
+}
+
+static char *tcp_ip_name(IP32bit ip)
+{
+	if(ip == NULL_IP) return 0;
+	struct hostent *hp = gethostbyaddr((char *)&ip, 4, AF_INET);
+	if (!hp) return 0;
+	return hp->h_name;
+}
+
+static char *tcp_dots_name(const char *s)
+{
+	return tcp_ip_name(tcp_dots_ip(s));
+}
+
+static int ircDial(char *host, int port)
+{
+	static struct addrinfo hints;
+	int srv;
+	struct timeval tv;
+	struct linger ling;
+	static int yes = 1;
+	static int no = 0;
+	struct addrinfo *res, *r;
+	char portbuf[12];
+	sprintf(portbuf, "%d", port);
+	memset(&hints, 0, sizeof hints);
+	hints.ai_family = AF_UNSPEC;
+	hints.ai_socktype = SOCK_STREAM;
+	if(getaddrinfo(host, portbuf, &hints, &res) != 0) {
+		setError(MSG_IdentifyHost, host);
+		return -1;
+	}
+	for(r = res; r; r = r->ai_next) {
+		if((srv = socket(r->ai_family, r->ai_socktype, r->ai_protocol)) == -1)
+			continue;
+
+// set some socket options.
+// Timer is intended to restrict connect, and shouldn't affect anything else.
+// In case you type in a bad domain or the server is down.
+// My reads are nonblocking, and I do the actual read
+// when I know there is something there.
+// Writes should go out quickly, I don't know why they wouldn't.
+// This code, from sic, tries all ip addresses for the domain, so time adds up.
+// I'll make the timeout relatively short.
+// Note you can hit ^c and it will not continue to the next ip address.
+		tv.tv_sec = 4;
+		tv.tv_usec = 0;
+		setsockopt(srv, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv));
+		setsockopt(srv, SOL_SOCKET, SO_SNDTIMEO, &tv, sizeof(tv));
+
+// we shouldn't be closing a socket unless we've finished the session.
+// If we are closing it for any other reason, if must be a failure leg.
+// May as well discard any pending data and close it quickly.
+		ling.l_onoff = 0;
+		ling.l_linger = 0;
+		fcntl(srv, F_SETFD, FD_CLOEXEC);
+		setsockopt(srv, SOL_SOCKET, SO_LINGER, (char *)&ling, sizeof(ling));
+
+		setsockopt(srv, SOL_SOCKET, SO_REUSEADDR, (char *)&yes, sizeof(yes));
+// I feel better setting REUSEPORT, but this option isn't available
+// on every system.  I hope it is the default.
+#ifdef SO_REUSEPORT
+		setsockopt(srv, SOL_SOCKET, SO_REUSEPORT, (char *)&yes, sizeof(yes));
+#endif
+// for now I don't see any advantage in keeping sockets warm.
+		setsockopt(srv, SOL_SOCKET, SO_KEEPALIVE, (char *)&no, sizeof(no));
+
+		if(connect(srv, r->ai_addr, r->ai_addrlen) == 0)
+			break;
+		close(srv);
+		if(intFlag) { r = 0; break; }
+	}
+	freeaddrinfo(res);
+
+	if(!r) {
+		setError(MSG_WebConnect, host);
+		return -1;
+	}
+
+// now that we're connected, up the timout to 10 seconds.
+	tv.tv_sec = 10;
+	tv.tv_usec = 0;
+	setsockopt(srv, SOL_SOCKET, SO_RCVTIMEO, &tv, sizeof(tv));
+	setsockopt(srv, SOL_SOCKET, SO_SNDTIMEO, &tv, sizeof(tv));
+
+	return srv;
+}
+
+bool ircSetup(char *line)
+{
+	int cxin, cxout, fd;
+	Window *win, *wout;
+	char *domain, *nick, *password, *join, *p;
+	int port = 6667;
+	FILE *f;
+	int l;
+
+	line += 3;
+	if(!*line) goto usage;
+	spaceCrunch(line, true, false);
+	if(!isdigit(*line)) goto usage;
+	cxin = strtol(line, &line, 10);
+	if(cxin < 0 || *line != ' ') goto usage;
+	++line;
+	if(!isdigit(*line)) goto usage;
+	cxout = strtol(line, &line, 10);
+	if(cxout < 0 || *line != ' ') goto usage;
+	++line;
+	if(!cxin || !cxout) {
+		setError(MSG_Session0);
+		return false;
+	}
+	if(cxin >= MAXSESSION) {
+		setError(MSG_SessionHigh, cxin, MAXSESSION - 1);
+		return false;
+	}
+	if(cxout >= MAXSESSION) {
+		setError(MSG_SessionHigh, cxout, MAXSESSION - 1);
+		return false;
+	}
+	if(cxin == cxout) {
+		setError(MSG_IrcCompat, cxout);
+		return false;
+	}
+	win = sessionList[cxin].lw;
+	if(!win) {
+		sideBuffer(cxin, emptyString, 0, 0);
+		win = sessionList[cxin].lw;
+	} else if(win->sqlMode | win->binMode | win->dirMode | win->browseMode | win->irciMode | win->ircoMode) {
+		setError(MSG_IrcCompat, cxin);
+		return false;
+	}
+	wout = sessionList[cxout].lw;
+	if(!wout) {
+		sideBuffer(cxout, emptyString, 0, 0);
+		wout = sessionList[cxout].lw;
+	} else if(wout->sqlMode | wout->binMode | wout->dirMode | wout->browseMode | wout->irciMode) {
+		setError(MSG_IrcCompat, cxout);
+		return false;
+	}
+	if(!isalpha(*line)) goto usage;
+	domain = line;
+	nick = ircSkip(line, ' ');
+	if(!*nick) goto usage;
+	join = ircSkip(nick, ' ');
+	if(!*join) join = 0;
+	password = ircSkip(nick, ':');
+	if(!*password) password = 0;
+	p = ircSkip(domain, ':');
+	if(*p) {
+		port = stringIsNum(p);
+		if(port < 0) {
+			setError(MSG_BadPort);
+			return false;
+		}
+	}
+
+	win->ircChannels = false;
+	l = strlen(domain);
+	if(domain[l-1] == '+')
+		domain[--l] = 0, win->ircChannels = true;
+
+	fd = ircDial(domain, port);
+	if(fd < 0) return false;
+	win->ircF = f = fdopen(fd, "r+");
+	win->irciMode = true;
+	wout->ircoMode = true;
+	wout->ircCount++;
+	win->ircOther = cxout;
+	win->ircNick = cloneString(nick);
+	nzFree(win->f0.hbase);
+	win->f0.hbase = cloneString(domain);
+	ircSetChannel(win, 0);
+
+	// login
+	if(password) {
+		ircSend(f, "PASS %s", password);
+		debugPrint(4, "PASS %s", password);
+	}
+	ircSend(f, "NICK %s", nick);
+	debugPrint(4, "NICK %s", nick);
+	ircSend(f, "USER %s localhost %s :%s", nick, domain, nick);
+	debugPrint(4, "USER %s localhost %s :%s", nick, domain, nick);
+	if(join) {
+		ircSend(f, "JOIN %s", join);
+		debugPrint(4, "JOIN %s", join);
+		ircSetChannel(win, join);
+	}
+	fflush(f);
+	setbuf(f, NULL);
+// And hopefully that worked.
+	return true;
+
+usage:
+	setError(MSG_IrcUsage);
+	return false;
+}
+
+/*********************************************************************
+irc is based on polling, just another interval timer, like javascript timers.
+This allows us to reuse that machinery.
+Cool, but what if edbrowse is locked up, they send ping, we don't check
+right away, and don't send pong. The connection drops. Oops!
+That can't happen, can it?
+If the user browses a complicated js file, and we are lost in the js
+engine, or busy downloading many css files, it could.
+Well, does it hurt when you do that?
+Then don't do that!
+It's best to have an edbrowse process just for your irc channels.
+But, if you use readline for input, and type one character,
+control passes to readline(), and I can't change that.
+edbrowse stops until you finish typing your line.
+If ping comes in, and you are composing your sonnet, and I don't pong back
+in time, there goes the connection.
+Well, why not put this irc fetch stuff in a background thread?
+Because almost nothing in edbrowse is threadsafe.
+Downloading files yes, I wrote it that way, but not the bufferstuff.
+If you change anything in a buffer, and irc adds something to its output
+buffer at the same time, bad things happen.  BOOM!
+It would take a lot of work to go down this path, but the one I'm on
+is also a bit dangerous.
+Use the unix timer to wake up every 3 seconds and poll irc,
+but only while readline() has control.
+This should get around the threadsafe issues.
+Only one "thread" is managing buffers at a time. We hope.
+*********************************************************************/
+
+#include <signal.h>
+
+static volatile bool ircAlarming;
+static void ircAlarm(int n)
+{
+	ircAlarming = true;
+	ircRead();
+	signal(SIGALRM, ircAlarm);
+	alarm(3);
+	ircAlarming = false;
+}
+
+void ircReadlineControl(void)
+{
+	signal(SIGALRM, ircAlarm);
+	alarm(3);
+}
+
+void ircReadlineRelease(void)
+{
+// This code downgrades a dangerous race condition from pretty darn unlikely
+// to nearly impossible - though it it is still possible.
+	while(ircAlarming)   ;
+// order is important here
+	alarm(0);
+	signal(SIGALRM, SIG_DFL);
+}
+
