@@ -13,7 +13,7 @@ Frame *cf;
 int gfsn; // global frame sequence number
 const char *progname;
 const char eol[] = "\r\n";
-const char *version = "3.8.4";
+const char *version = "3.8.5";
 char *changeFileName;
 char *configFile, *addressFile, *cookieFile, *emojiFile;
 char *mailDir, *mailUnread, *mailStash, *mailReply;
@@ -34,6 +34,7 @@ int verifyCertificates = 1;
 char *sslCerts, *pubKey;
 int localAccount, maxAccount;
 struct MACCOUNT accounts[MAXACCOUNT];
+bool flow = true; // send mail with format=flowed
 int maxMime;
 struct MIMETYPE mimetypes[MAXMIME];
 static struct DBTABLE dbtables[MAXDBT];
@@ -43,7 +44,7 @@ time_t intStart;
 bool ismc, isimap, passMail;
 // next two variables work around curl bug 7284
 bool inInput, inInitFunction, listNA;
-int fileSize;
+long long fileSize;
 char *dbarea, *dblogin, *dbpw;	// to log into the database
 bool fetchBlobColumns;
 bool caseInsensitive, searchStringsAll, searchWrap = true, ebre = true;
@@ -418,28 +419,11 @@ const char *mailRedirect(const char *to, const char *from,
 static void setupEdbrowseTempDirectory(void)
 {
 	int userid;
-#ifdef DOSLIKE
-	int l;
-	char *a;
-	ebTempDir = getenv("TEMP");
-	if (!ebTempDir) {
-		i_printf(MSG_NoEnvVar, "TEMP");
-		nl();
-		exit(1);
-	}
-// put /edbrowse on the end
-	l = strlen(ebTempDir);
-	a = allocString(l + 10);
-	sprintf(a, "%s/edbrowse", ebTempDir);
-	ebTempDir = a;
-	userid = 0;
-#else
 	ebTempDir = getenv("TMPDIR");
 	if (!ebTempDir) {
 		ebTempDir="/tmp/.edbrowse";
 	}
 	userid = geteuid();
-#endif
 
 // On a multiuser system, mkdir /tmp/.edbrowse at startup,
 // by root, and then chmod 1777
@@ -452,10 +436,8 @@ static void setupEdbrowseTempDirectory(void)
 			ebTempDir = 0;
 			return;
 		}
-#ifndef DOSLIKE
 // yes, we called mkdir with 777 above, but that was cut by umask.
 		chmod(ebTempDir, MODE_rwx);
-#endif
 	}
 // make room for user ID on the end
 	ebUserDir = allocMem(strlen(ebTempDir) + 30);
@@ -471,17 +453,6 @@ static void setupEdbrowseTempDirectory(void)
 	}
 }
 
-/*\ MSVC Debug: May need to provide path to 3rdParty DLLs, like
- *  set PATH=F:\Projects\software\bin;%PATH% ...
-\*/
-
-/* I'm not going to expand wild card arguments here.
- * I don't need to on Unix, and on Windows there is a
- * setargv.obj, or something like that, that performs the expansion.
- * I'll assume you have folded that object into libc.lib.
- * So now you can edit *.c, on any operating system,
- * and it will do the right thing, with no work on my part. */
-
 static void loadReplacements(void);
 
 int main(int argc, char **argv)
@@ -489,13 +460,13 @@ int main(int argc, char **argv)
 	int cx, account;
 	bool rc, doConfig = true, autobrowse = false;
 	bool dofetch = false, domail = false, setDebugOpt = false;
+	char *firstFile = 0;
+	int firstFilePosition = 0, k;
 	static char agent0[64] = "edbrowse/";
 
-#ifndef _MSC_VER		// port setlinebuf(stdout);, if required...
 // In case this is being piped over to a synthesizer, or whatever.
 	if (fileTypeByHandle(fileno(stdout)) != 'f')
 		setlinebuf(stdout);
-#endif // !_MSC_VER
 
 	selectLanguage();
 	setHTTPLanguage(eb_language);
@@ -503,40 +474,12 @@ int main(int argc, char **argv)
 // bring HISTCONTROL down to edbrowse
 	setHistcontrol();
 
-/* Establish the home directory, and standard edbrowse files thereunder. */
+// Establish the home directory, and standard edbrowse files thereunder.
 	home = getenv("HOME");
-#ifdef _MSC_VER
-	if (!home) {
-		home = getenv("APPDATA");
-		if (home) {
-			char *ebdata = (char *)allocMem(ABSPATH);
-			sprintf(ebdata, "%s\\edbrowse", home);
-			if (fileTypeByName(ebdata, 0) != 'd') {
-				FILE *fp;
-				char *cfgfil;
-				if (mkdir(ebdata, 0700)) {
-					i_printfExit(MSG_NotHome);	// TODO: more appropriate exit message...
-				}
-				cfgfil = (char *)allocMem(ABSPATH);
-				sprintf(cfgfil, "%s\\.ebrc", ebdata);
-				fp = fopen(cfgfil, "w");
-				if (fp) {
-					fwrite(ebrc_string, 1,
-					       strlen(ebrc_string), fp);
-					fclose(fp);
-				}
-				i_printfExit(MSG_Personalize, cfgfil);
 
-			}
-			home = ebdata;
-		}
-	}
-#endif // !_MSC_VER
-
-/* Empty is the same as missing. */
+// Empty is the same as missing
 	if (home && !*home)
 		home = 0;
-/* I require this, though I'm not sure what this means for non-Unix OS's */
 	if (!home)
 		i_printfExit(MSG_NotHome);
 	if (fileTypeByName(home, 0) != 'd')
@@ -592,7 +535,7 @@ int main(int argc, char **argv)
 	ttySaveSettings();
 	initializeReadline();
 
-	loadReplacements();
+	loadReplacements(); // for debugging a snapshot
 
 	if (argc && stringEqual(argv[0], "-c")) {
 		if (argc == 1) {
@@ -670,7 +613,7 @@ int main(int argc, char **argv)
 		}
 
 		i_printfExit(MSG_Usage);
-	}			/* options */
+	}			// options
 
 	srand(time(0));
 
@@ -753,14 +696,49 @@ int main(int argc, char **argv)
 	js_main();
 
 // This sanity check on number of files assumes they are all files,
-// not functions to execute.
+// not commands to execute.
 	if(argc >= MAXSESSION)
 		i_printfExit(MSG_ManyOpen, MAXSESSION - 1);
 
 	cx = 0;
+// where is the first file?
+	for(k = 0; k < argc; ++k) {
+		if(argv[k][0] != '+') {
+			firstFile = argv[k];
+			firstFilePosition = k;
+			break;
+		}
+	}
+
+	if(firstFile && firstFilePosition) {
+// move +commands to after the first file
+		memmove(argv + 1, argv, firstFilePosition * sizeof(char*));
+		argv[0] = firstFile;
+	}
+
+	if (!cx && !firstFile) {		// no files
+// create an empty buffer
+		++cx;
+		cxSwitch(cx, false);
+		inInitFunction = setDebugOpt;
+		runEbFunction("init");
+		inInitFunction = false;
+		if(debugLevel >= 1)
+			i_puts(MSG_Ready);
+	}
+
 	while (argc) {
 		char *file = *argv;
 		char *file2 = 0;	// will be allocated
+
+// edbrowse command as argument
+		if (file[0] == '+') {
+			debugPrint(3, "+ %s", file + 1);
+			edbrowseCommand(file + 1, false);
+			++argv, --argc;
+			continue;
+		}
+
 		++cx;
 		if (cx == MAXSESSION)
 			i_printfExit(MSG_ManyOpen, MAXSESSION - 1);
@@ -771,9 +749,8 @@ int main(int argc, char **argv)
 			inInitFunction = false;
 		}
 
-// function on the command line
-		if (file[0] == '<') {
-			runEbFunction(file + 1);
+		if(!*file) { // empty string, empty buffer
+			debugPrint(1, "0");
 			++argv, --argc;
 			continue;
 		}
@@ -800,7 +777,7 @@ int main(int argc, char **argv)
 				file2[0] = 'b';
 				file2[1] = ' ';
 				if (runCommand(file2))
-					debugPrint(1, "%d", fileSize);
+					debugPrint(1, "%lld", fileSize);
 				else
 					showError();
 			}
@@ -816,7 +793,7 @@ int main(int argc, char **argv)
 				cw->sqlMode = true;
 			rc = readFileArgv(file, 0, 0);
 			if (fileSize >= 0)
-				debugPrint(1, "%d", fileSize);
+				debugPrint(1, "%lld", fileSize);
 			fileSize = -1;
 			if (!rc) {
 				showError();
@@ -839,7 +816,7 @@ int main(int argc, char **argv)
 			    && ((cf->mt && cf->mt->outtype)
 				|| isBrowseableURL(cf->fileName))) {
 				if (runCommand("b")) {
-					debugPrint(1, "%d", fileSize);
+					debugPrint(1, "%lld", fileSize);
 					if(newhash) {
 						set_location_hash(newhash);
 						if(!jump2anchor(0, newhash))
@@ -853,16 +830,8 @@ int main(int argc, char **argv)
 
 		nzFree(file2);
 		++argv, --argc;
-	}			/* loop over files */
-	if (!cx) {		/* no files */
-		++cx;
-		cxSwitch(cx, false);
-		inInitFunction = setDebugOpt;
-		runEbFunction("init");
-		inInitFunction = false;
-		if(debugLevel >= 1)
-			i_puts(MSG_Ready);
-	}
+	}			// loop over files
+
 	if (cx > 1)
 		cxSwitch(1, false);
 
@@ -918,9 +887,46 @@ static const char *balance(const char *ip, int direction)
 	return ip;
 }
 
+static const char *ip_ptr;
+static const char **endl_ptr;
+static const char **args_ptr;
+static int *argl_ptr;
+
+static char *substituteArgs(void)
+{
+	const char *ip = ip_ptr;
+	const char *endl = *endl_ptr;
+	const char **args = args_ptr;
+	int *argl = argl_ptr;
+		int j, l = endl - ip;
+	const char *s;
+	char *new, *t;
+// compute length of line, then build the line
+	for (s = ip; s < endl; ++s)
+		if (*s == '~' && isdigitByte(s[1]))
+			l += argl[s[1] - '0'];
+	t = new = allocMem(l + 1);
+	for (s = ip; s < endl; ++s) {
+		if (*s == '~' && isdigitByte(s[1])) {
+			j = *++s - '0';
+			if (!args[j]) {
+				setError(MSG_NoArgument, j);
+				nzFree(new);
+				return 0;
+			}
+			strcpy(t, args[j]);
+			t += argl[j];
+			continue;
+		}
+		*t++ = *s;
+	}
+	*t = 0;
+	return new;
+}
+
 #define MAXNEST 20		// nested blocks
-/* Run an edbrowse function, as defined in the config file. */
-/* This function must be reentrant. */
+// Run an edbrowse function, as defined in the config file.
+// This function must be reentrant, as a script can call another script.
 int runEbFunction(const char *line)
 {
 	char *linecopy = cloneString(line);
@@ -930,7 +936,7 @@ int runEbFunction(const char *line)
 	int argl[10];		/* lengths of args */
 	const char *s;
 	char *t, *new;
-	int l, nest;
+	int nest;
 	unsigned j;
 	const char *ip;		// think instruction pointer
 	const char *endl;	// end of line to be processed
@@ -942,7 +948,7 @@ int runEbFunction(const char *line)
 
 // local copies of settings, to restore after function runs.
 	struct {
-		bool rl, endm, lna, H, ci, sg, su8, sw, ebre, bd, iu, hf, hr, vs, sr, can, ftpa, bg, jsbg, js, showall, pg, fbc, ls_reverse, fllo, dno;
+		bool rl, endm, lna, H, ci, sg, su8, sw, ebre, bd, iu, hf, hr, vs, sr, can, ftpa, bg, jsbg, js, showall, pg, fbc, ls_reverse, fllo, dno, flow;
 		uchar dw, ls_sort;
 		char lsformat[12], showProgress;
 		char *currentAgent;
@@ -990,6 +996,7 @@ int runEbFunction(const char *line)
 		save.timerspeed = timerspeed;
 		save.dw = dirWrite;
 		save.dno = dno;
+		save.flow = flow;
 		save.ls_sort = ls_sort;
 		strcpy(save.lsformat, lsformat);
 		save.showProgress = showProgress;
@@ -1011,7 +1018,7 @@ int runEbFunction(const char *line)
 		save.H = helpMessagesOn;
 		save.pg = pluginsOn;
 		save.hf = showHiddenFiles;
-		save.showall = showHover;
+		save.showall = showall;
 		save.endm = endMarks;
 		save.sg = searchStringsAll;
 		save.ci = caseInsensitive;
@@ -1119,29 +1126,14 @@ ahead:
 		if (!ok && nofail)
 			goto soft_fail;
 
-/* compute length of line, then build the line */
-		l = endl - ip;
-		for (s = ip; s < endl; ++s)
-			if (*s == '~' && isdigitByte(s[1]))
-				l += argl[s[1] - '0'];
-		t = new = allocMem(l + 1);
-		for (s = ip; s < endl; ++s) {
-			if (*s == '~' && isdigitByte(s[1])) {
-				j = *++s - '0';
-				if (!args[j]) {
-					setError(MSG_NoArgument, j);
-					nzFree(new);
-					goto done;
-				}
-				strcpy(t, args[j]);
-				t += argl[j];
-				continue;
-			}
-			*t++ = *s;
-		}
-		*t = 0;
+		ip_ptr = ip;
+		endl_ptr = &endl;
+		args_ptr = args;
+		argl_ptr = argl;
+		new = substituteArgs();
+		if(!new) goto done; // substitution failed
 
-/* Here we go! */
+// Here we go!
 		debugPrint(3, "< %s", new);
 		if (cw->mustrender) {
 			time_t now;
@@ -1172,6 +1164,7 @@ done:
 		timerspeed = save.timerspeed;
 		dirWrite = save.dw;
 		dno = save.dno;
+		flow = save.flow;
 		ls_sort = save.ls_sort;
 		strcpy(lsformat, save.lsformat);
 		showProgress = save.showProgress;
@@ -1192,7 +1185,7 @@ done:
 		helpMessagesOn = save.H;
 		pluginsOn = save.pg;
 		showHiddenFiles = save.hf;
-		showHover = save.showall;
+		showall = save.showall;
 		endMarks = save.endm;
 		searchStringsAll = save.sg;
 		caseInsensitive = save.ci;
@@ -1209,6 +1202,30 @@ soft_fail:
 	goto done;
 }
 
+// so we can append text from within a script, like a here document
+const char *getInputLineFromScript(void)
+{
+	const char *endl = *endl_ptr;
+	const char *ip = endl + 1;
+	char *new;
+// the script could run out before . terminates the input
+	if(!*ip) return ".\n";
+	endl = strchr(ip, '\n');
+	*endl_ptr = endl;
+	ip_ptr = ip;
+	if(ip[0] == '.' && ip[1] == '\n') return ip;
+	new =  substituteArgs();
+	if(!new) return 0; // error
+// corner case, becomes . after substitution
+	if(new[0] == '.' && new[1] == 0) {
+		free(new);
+		return "*.@sub~$`corner";
+	}
+// switch from c string to perl string
+	new[strlen(new)] = '\n';
+	return new;
+}
+
 struct DBTABLE *findTableDescriptor(const char *sn)
 {
 	int i;
@@ -1487,7 +1504,7 @@ bool readConfigFile(void)
 
 	unreadConfigFile();
 
-	if (!fileIntoMemory(configFile, &buf, &buflen)) {
+	if (!fileIntoMemory(configFile, &buf, &buflen, 0)) {
 		i_printf(MSG_NoConfig, configFile);
 		return 0;
 	}
@@ -1841,8 +1858,7 @@ inside:
 
 		case 37:	/* imap fetch limit */
 			imapfetch = atoi(v);
-			if (imapfetch <= 10)
-				imapfetch = 10;
+			if (imapfetch < 5) imapfetch = 5;
 // there isn't an upper limit when set interactively, so idk
 			if (imapfetch >= 1000)
 				imapfetch = 1000;
@@ -1893,7 +1909,7 @@ inside:
 		case 44: // include
 			v = envFileAlloc(v);
 			if(!v) continue;
-			if(!fileIntoMemory(v, &incbuf, &inclen)) {
+			if(!fileIntoMemory(v, &incbuf, &inclen, 0)) {
 				showError();
 				setError(-1);
 				nzFree(v);
