@@ -12,7 +12,7 @@ blank spaces for messages no longer used.
 Each file lang/msg-* must have exactly this many lines.
 *********************************************************************/
 
-#define EdbrowseMessageCount 700
+#define EdbrowseMessageCount 800
 
 // English
 extern const char *msg_en[], ebrc_en[], qrg_en[];
@@ -227,7 +227,7 @@ enum {
 	MSG_DomainEmpty,
 	MSG_NoWriteURL,
 	MSG_WriteDB,
-	MSG_WriteEmpty,
+	MSG_IrcCommand,
 	MSG_NoCreate2,
 	MSG_NoWrite2,
 	MSG_SessionBackground,
@@ -376,12 +376,12 @@ enum {
 	MSG_NoRename,
 	MSG_InputNull2,
 	MSG_InputNewline2,
-	MSG_NoInputFields,
-	MSG_NoLinks,
-	MSG_NoButtons,
-	MSG_ManyInputFields,
-	MSG_ManyLinks,
-	MSG_ManyButtons,
+	MSG_notused279,
+	MSG_notused280,
+	MSG_notused281,
+	MSG_notused282,
+	MSG_notused283,
+	MSG_notused284,
 	MSG_WebRead,
 	MSG_BadURL,
 	MSG_DownSuccess,
@@ -436,14 +436,14 @@ enum {
 	MSG_ProgressDots,
 	MSG_SigRegular,
 	MSG_SigAccess,
-	MSG_MailBinary,
+	MSG_MailIrc,
 	MSG_AttAlternate,
 	MSG_RecipMany,
 	MSG_ABMissing,
 	MSG_ABNoAlias2,
 	MSG_RecipNone,
 	MSG_AttSessionEmpty,
-	MSG_notused346,
+	MSG_ReIrc,
 	MSG_AttRegular,
 	MSG_AttEmpty2,
 	MSG_LineDelete1,
@@ -793,4 +793,24 @@ enum {
 	MSG_DirNamesOnlyOn,
 	MSG_DebugTagsOff,
 	MSG_DebugTagsOn,
+	MSG_FormatFlowedOff,
+	MSG_FormatFlowedOn,
+	MSG_AudioIrc,
+	MSG_BreakIrc,
+	MSG_IrcRename,
+	MSG_IrcEntire,
+	MSG_IrcUsage,
+	MSG_IrcCompat,
+	MSG_DebugLayOff,
+	MSG_DebugLayOn,
+	MSG_NoInputFields,
+	MSG_NoLinks,
+	MSG_NoButtons,
+	MSG_NoImages,
+	MSG_hold710,
+	MSG_ManyInputFields,
+	MSG_ManyLinks,
+	MSG_ManyButtons,
+	MSG_ManyImages,
+	MSG_hold715,
 };
