@@ -10,6 +10,7 @@
 #define MAXRECAT 100		// max number of recipients or attachments
 #define MAXMSLINE 1024		// max mail server line
 #define LONGLINELIMIT 76
+#define LONGWORDLIMIT 998
 
 static char serverLine[MAXMSLINE];
 static bool doSignature;
@@ -41,7 +42,7 @@ bool loadAddressBook(void)
 	nzFree(addressList);
 	addressList = 0;
 	nads = 0;
-	if (!fileIntoMemory(addressFile, &buf, &buflen))
+	if (!fileIntoMemory(addressFile, &buf, &buflen, 0))
 		return false;
 	bufend = buf + buflen;
 
@@ -274,11 +275,11 @@ bool *long_p)
 {
 	char *buf;
 	char c;
-	bool longline;
+	bool longline, longword, cr, endlinespace, flowed;
 	char *s, *t, *v;
-	char *ct, *ce;		/* content type, content encoding */
+	char *ct, *ce;		// content type, content encoding
 	int buflen, i, cx;
-	int nacount, nullcount, linelength;
+	int nacount, nullcount, linelength, wordlength;
 
 	debugPrint(5, "subject at line %d", ismail);
 	if (ismail < 0) {
@@ -308,7 +309,7 @@ empty:
 			if (sessionList[cx].lw->f0.fileName)
 				file = sessionList[cx].lw->f0.fileName;
 		} else {
-			if (!fileIntoMemory(file, &buf, &buflen))
+			if (!fileIntoMemory(file, &buf, &buflen, 0))
 				return false;
 			if (!buflen) {
 				if (webform)
@@ -448,8 +449,8 @@ empty:
 	}
 
 /* Count the nonascii characters */
-	nacount = nullcount = linelength = 0;
-	longline = false;
+	nacount = nullcount = linelength = wordlength = 0;
+	longline = longword = cr = endlinespace = flowed = false;
 	for (t = buf, i = 0; i < buflen; ++i, ++t) {
 		c = *t;
 		if (c == '\0')
@@ -459,13 +460,25 @@ empty:
 		if (c == '\n') {
 			if(linelength > LONGLINELIMIT) longline = true;
 			linelength = 0;
-			continue;
-		}
+		} else {
 // measure length of line by utf8 characters
-		if(((uchar)c & 0xc0) != 0x80)
-			++linelength;
+			if(((uchar)c & 0xc0) != 0x80)
+				++linelength;
+		}
+		if (c == '\n' || c == '\r' || c == '\t' || c == ' ') {
+			if (wordlength > LONGWORDLIMIT) longword = true;
+			wordlength = 0;
+			if (c == '\r' && i < buflen - 1 && t[1] != '\n') cr = true;
+// If a line not at the end of a paragraph ends with a space,
+// consider using format=flowed.
+			if (c == ' ' && i + 2 < buflen &&
+			((t[1] == '\n' && t[2] != '\n') ||
+			(t[1] == '\r' && t[2] != '\r' && (t[2] != '\n' || (i + 3 < buflen && t[3] != '\r')))))
+				endlinespace = true;
+		} else ++wordlength;
 	}
 	if(linelength > LONGLINELIMIT) longline = true;
+	if (wordlength > LONGWORDLIMIT) longword = true;
 	debugPrint(5, "attaching %s length %d nonascii %d nulls %d longline %d",
 		   file, buflen, nacount, nullcount, longline);
 	nacount += nullcount;
@@ -478,16 +491,16 @@ empty:
 	if (!ct)
 		ct = "text/plain";
 
-/* Criteria for base64 encode.
- * files uploaded from a web form need not be encoded, unless they contain
- * nulls, which is a quirk of my slapped together software. */
+// base64 is almost obsolete, with networks and clients able to handle 8bit,
+// but I still have to use it if the data contains even a single null,
+// because edbrowse treats data as a string after this point.
+// I'll still use 64 if it looks binary, perhaps a vital executable,
+// rather than simple utf8 text.
+// Also use base64 for text attachments that include a carriage return
+// when using format=flowed and for all attachments when not using
+// format=flowed.
 
-	if ((!webform && buflen > 20 && nacount * 4 > buflen) ||
-	    nullcount) {
-		if (ismail) {
-			setError(MSG_MailBinary, file);
-			goto freefail;
-		}
+	if ((!ismail && (cr || !flow)) || nullcount || looksBinary((uchar*)buf, buflen)) {
 		s = base64Encode(buf, buflen, true);
 		nzFree(buf);
 		buf = s;
@@ -495,77 +508,105 @@ empty:
 		goto success;
 	}
 
-	if (!webform) {
-// Use qp for long lines, it doesn't hurt,
-// and when I send I copy lines into a static buffer of a fixed length.
-		if (longline) {
-			char *newbuf;
-			int l, colno = 0, space = 0;
-
-			newbuf = initString(&l);
-			v = buf + buflen;
-			for (s = buf; s < v; ++s) {
-				c = *s;
-/* do we have to =expand this character? */
-				if ((c < '\n' && c != '\t') ||
-				    c == '=' ||
-				    c == '\xff' ||
-				    ((c == ' ' || c == '\t') &&
-				     s < v - 1 && (s[1] == '\n' || s[1] == '\r'))) {
-					char expand[4];
-					sprintf(expand, "=%02X", (uchar) c);
-					stringAndString(&newbuf, &l, expand);
-					colno += 3;
-				} else {
-					stringAndChar(&newbuf, &l, c);
-					++colno;
-				}
-				if (c == '\n' || c == '\r') {
-					colno = space = 0;
-					continue;
-				}
-				if (c == ' ' || c == '\t')
-					space = l;
-				if (colno < 72)
-					continue;
-				if (s == v - 1)
-					continue;
-/* If newline's coming up anyways, don't force another one. */
-				if (s[1] == '\n' || s[1] == '\r')
-					continue;
-				i = l;
-				if (!space || space == i) {
-					stringAndString(&newbuf, &l, "=\n");
-					colno = space = 0;
-					continue;
-				}
-				colno = i - space;
-				stringAndString(&newbuf, &l, "**");	/* make room */
-				while (i > space) {
-					newbuf[i + 1] = newbuf[i - 1];
-					--i;
-				}
-				newbuf[space] = '=';
-				newbuf[space + 1] = '\n';
-				space = 0;
-			}	/* loop over characters */
-
-			nzFree(buf);
-			buf = newbuf;
-			ce = "quoted-printable";
-			goto success;
-		}
+	ce = (nacount ? "8bit" : "7bit");
+	if (webform || !(longline || (ismail && flow && endlinespace))) {
+		buf[buflen] = 0;
+		goto success;
 	}
 
-	buf[buflen] = 0;
-	ce = (nacount ? "8bit" : "7bit");
+// Use qp for long lines, it doesn't hurt,
+// and when I send I copy lines into a static buffer of a fixed length.
+
+	char *newbuf;
+	int l, colno = 0, space = 0;
+	if (ismail && flow && !longword) flowed = true;
+	else ce = "quoted-printable";
+	newbuf = initString(&l);
+	v = buf + buflen;
+	for (s = buf; s < v; ++s) {
+		c = *s;
+// With format=flowed, skip the space characters at the end of a paragraph
+// and double an = at the end of a line.
+		if (flowed && c == '=') {
+// skip past spaces
+			for (t = s + 1; t < v && *t == ' '; ++t)  ;
+			if(t == v || *t == '\r' || *t == '\n') {
+				stringAndString(&newbuf, &l, "==");
+				s = t - 1;
+				continue;
+			}
+		}
+		if (flowed && c == ' ') {
+// skip past spaces
+			for (t = s + 1; t < v && *t == ' '; ++t)  ;
+			if(t == v ||
+			((t[0] == '\r' || t[0] == '\n') && (t + 1 == v || t[1] == t[0] || (t[0] == '\n' && t[1] == '\r'))) ||
+			(t[0] == '\r' && (t + 1 == v || (t[1] == '\n' && (t + 2 == v || t[2] == '\r' || t[2] == '\n'))))) {
+				s = t - 1;
+				continue;
+			}
+		}
+// do we have to =expand this character?
+		if (!flowed &&
+		((((uchar)c < '\n' && c != '\t') ||
+		    c == '=' ||
+		    (uchar)c == '\xff' ||
+		    ((c == ' ' || c == '\t') &&
+		     s < v - 1 && (s[1] == '\n' || s[1] == '\r'))))) {
+			char expand[4];
+			sprintf(expand, "=%02X", (uchar) c);
+			stringAndString(&newbuf, &l, expand);
+			colno += 3;
+		} else {
+			stringAndChar(&newbuf, &l, c);
+// make a utf8 char look like 1
+			if((c&0x80) == 0 || (c&0xc0) == 0xc0) ++colno;
+			if (flowed && c == ' ' && (s[1] == '\n' || s[1] == '\r')) {
+// With format=flowed, when a space ends a line within a paragraph, add = so that we can flow the whole paragraph later.
+				stringAndChar(&newbuf, &l, '=');
+			}
+		}
+		if (c == '\n') {
+			colno = space = 0;
+			continue;
+		}
+		if (isspaceByte(c))
+			space = l;
+		if (colno < 72)
+			continue;
+		if (s == v - 1)
+			continue;
+// If newline's coming up anyways, don't force another one.
+		if (s[1] == '\n')
+			continue;
+		i = l;
+		if (!flowed && (!space || space == i)) {
+			stringAndString(&newbuf, &l, "=\n");
+			colno = space = 0;
+			continue;
+		}
+// a long line without spaces, like a long url
+		if(!space) continue;
+		colno = i - space;
+		stringAndString(&newbuf, &l, "**");	/* make room */
+		while (i > space) {
+			newbuf[i + 1] = newbuf[i - 1];
+			--i;
+		}
+		newbuf[space] = '=';
+		newbuf[space + 1] = '\n';
+		space = 0;
+	}	// loop over characters
+
+	nzFree(buf);
+	buf = newbuf;
 
 success:
 	debugPrint(5, "encoded %s %s length %d", ct, ce, strlen(buf));
 	*enc_p = ce;
 	*type_p = ct;
 	*data_p = buf;
-if(long_p) *long_p = longline;
+if(long_p) *long_p = flowed;
 	debugPrint(6, "%s", buf);
 	return true;
 
@@ -617,22 +658,28 @@ static char *messageTimeID(void)
 	return buf;
 }
 
-static void appendAttachment(const char *s, char **out, int *l, bool longline)
+static void appendAttachment(const char *s, char **out, int *l, bool flowed,
+const char *ct)
 {
 	const char *t;
 	int n;
 	int paraplus = -1;
+	bool ta; // text attachment
+	bool cr; // cr was at the end
+// check to see it's not a base64 attachment
+	ta = (ct && *ct != 'b');
 	while (*s) {		/* another line */
 		t = strchr(s, '\n');
 		if (!t)
 			t = s + strlen(s);
 		n = t - s;
+		cr = false;
 		if (n && t[-1] == '\r')
-			--n;
+			--n, cr = true;
 		if(!n) {
 			paraplus = -1;
 		} else {
-			if(longline && paraplus < 0) {
+			if(flowed && paraplus < 0) {
 				paraplus = 0;
 // does this paragraph have any long lines; should we append +
 				const char *u, *v, *x;
@@ -646,17 +693,19 @@ static void appendAttachment(const char *s, char **out, int *l, bool longline)
 				if(u && (!v || v > u)) paraplus = 1;
 			}
 			memcpy(serverLine, s, n);
-// if format=flowed, put spaces on the end of lines. Experimental!
-			if(longline && paraplus > 0 && serverLine[n-1] != '=' &&
-			serverLine[n-1] != ' ' &&
-			t[0] && t[1] &&
-			t[1] != '\n' && t[1] != '\r')
-				serverLine[n++] = ' ';
-// We might be able to remove = from a line that ends in space =
-// format=flowed will fix everything up anyways.
+// With format=flowed, put spaces on the end of lines in paragraphs containing
+// a long line and remove = at the end of lines. */
+			if(flowed && paraplus > 0) {
+				if (serverLine[n-1] == '=')
+					--n;
+				if ((!n || serverLine[n-1] != ' ') &&
+				t[0] && t[1] &&
+				t[1] != '\n' && t[1] != '\r')
+					serverLine[n++] = ' ';
+			}
 		}
 		serverLine[n] = 0;
-		strcat(serverLine, eol);
+		strcat(serverLine, ((ta && !cr) ? "\n" : eol));
 		stringAndString(out, l, serverLine);
 		if (*t)
 			++t;
@@ -876,7 +925,7 @@ sendMail(int account, const char **recipients, const char *body,
 	char *out = 0;
 	bool sendmail_success = false;
 	bool mustmime = false;
-	bool firstrec, longline;
+	bool firstrec, flowed;
 	const char *ct, *ce;
 	char *encoded = 0;
 
@@ -995,7 +1044,7 @@ sendMail(int account, const char **recipients, const char *body,
 		}
 	}			// loop over attachments
 
-	if (!encodeAttachment(body, subjat, false, &ct, &ce, &encoded, &longline))
+	if (!encodeAttachment(body, subjat, false, &ct, &ce, &encoded, &flowed))
 		return false;
 	if (ce[0] == 'q')
 		mustmime = true;
@@ -1070,7 +1119,7 @@ sendMail(int account, const char **recipients, const char *body,
 		sprintf(serverLine,
 			"Content-Type: %s%s%s%sContent-Transfer-Encoding: %s%s%s",
 			ct, charsetString(ct, ce),
-			(longline ? "; format=flowed" : ""), eol,
+			(flowed ? "; format=flowed; delsp=no" : ""), eol,
 			ce, eol, eol);
 		stringAndString(&out, &j, serverLine);
 	} else {
@@ -1086,13 +1135,13 @@ this format, some or all of this message may not be legible.\r\n\r\n--");
 		sprintf(serverLine,
 			"%sContent-Type: %s%s%s%sContent-Transfer-Encoding: %s%s%s",
 			eol, ct, charsetString(ct, ce),
-			(longline ? "; format=flowed" : ""), eol,
+			(flowed ? "; format=flowed; delsp=no" : ""), eol,
 			ce, eol, eol);
 		stringAndString(&out, &j, serverLine);
 	}
 
 /* Now send the body, line by line. */
-	appendAttachment(encoded, &out, &j, longline);
+	appendAttachment(encoded, &out, &j, flowed, NULL);
 	nzFree(encoded);
 	encoded = 0;
 
@@ -1132,7 +1181,7 @@ this format, some or all of this message may not be legible.\r\n\r\n--");
 				"%sContent-Transfer-Encoding: %s%s%s", eol, ce,
 				eol, eol);
 			stringAndString(&out, &j, serverLine);
-			appendAttachment(encoded, &out, &j, false);
+			appendAttachment(encoded, &out, &j, false, ct);
 			nzFree(encoded);
 			encoded = 0;
 		}		/* loop over attachments */
@@ -1186,6 +1235,10 @@ bool sendMailCurrent(int sm_account, bool dosig)
 		setError(MSG_MailDB);
 		return false;
 	}
+	if (cw->irciMode | cw->ircoMode) {
+		setError(MSG_MailIrc);
+		return false;
+	}
 	if (cw->dirMode) {
 		setError(MSG_MailDir);
 		return false;
