@@ -3,22 +3,19 @@
 #include "eb.h"
 
 #include <dirent.h>
-#ifdef DOSLIKE
-#include <dos.h>
-#else
 #include <termios.h>
 #include <unistd.h>
 #include <glob.h>
 #include <pwd.h>
 #include <grp.h>
-#endif
 
 char emptyString[] = "";
 bool showHiddenFiles, isInteractive;
 int debugLevel = 1;
-bool debugClone, debugEvent, debugThrow, debugCSS;
 FILE *debugFile = NULL;
 char *debugFileName;
+bool debugClone, debugEvent, debugThrow, debugCSS;
+bool debugLayout;
 bool demin = false;
 bool gotimers = true;
 bool uvw;
@@ -603,7 +600,7 @@ void errorPrint(const char *msg, ...)
 
 	if (bailflag)
 		exit(bailflag);
-}				/* errorPrint */
+}
 
 void debugPrint(int lev, const char *msg, ...)
 {
@@ -629,7 +626,7 @@ void debugPrint(int lev, const char *msg, ...)
 	}
 	if (lev == 0 && !memcmp(msg, "warning", 7))
 		eeCheck();
-}				/* debugPrint */
+}
 
 void setDebugFile(const char *name)
 {
@@ -643,11 +640,7 @@ void setDebugFile(const char *name)
 	debugFileName = cloneString(name);
 	debugFile = fopen(name, "w");
 	if (debugFile) {
-#ifndef _MSC_VER		// port setlinebuf(debugFile);, if required...
 		setlinebuf(debugFile);
-#else
-		;
-#endif // !_MSC_VER
 	} else
 		printf("cannot create %s\n", name);
 }
@@ -655,7 +648,7 @@ void setDebugFile(const char *name)
 void nl(void)
 {
 	eb_puts("");
-}				/* nl */
+}
 
 /* Turn perl string into C string, and complain about nulls. */
 int perl2c(char *t)
@@ -668,10 +661,10 @@ int perl2c(char *t)
 	}
 	*t = 0;			/* now it's a C string */
 	return n;		/* number of nulls */
-}				/* perl2c */
+}
 
 /* The length of a perl string includes its terminating newline */
-unsigned pstLength(const pst s)
+unsigned pstLength(const uchar *s)
 {
 	const uchar *t;
 	if (!s)
@@ -682,12 +675,11 @@ unsigned pstLength(const pst s)
 	return t + 1 - s;
 }
 
-pst clonePstring(const pst s)
+pst clonePstring(const uchar *s)
 {
 	pst t;
 	unsigned len;
-	if (!s)
-		return s;
+	if (!s) return 0;
 	len = pstLength(s);
 	t = (pst) allocMem(len);
 	memcpy(t, s, len);
@@ -699,33 +691,58 @@ void copyPstring(pst s, const pst t)
 {
 	int len = pstLength(t);
 	memcpy(s, t, len);
-}				/* copyPstring */
+}
 
-/*
- * fdIntoMemory reads data from a file descriptor, until EOF is reached.
- * It works even if we don't know the size beforehand.
- * We can now use it to read /proc files, pipes, and stdin.
- * This solves an outstanding issue, and it is needed for forthcoming
- * functionality, such as edpager.
- */
-bool fdIntoMemory(int fd, char **data, int *len)
+int comparePstring(const uchar * s, const uchar * t)
 {
-	int length, n;
+	uchar c, d;
+	while(true) {
+		c = *s, d = *t;
+		if(c == d) {
+			if(c == '\n') return 0;
+			 ++s, ++t;
+			continue;
+		}
+		if(c == '\n') return -1;
+		if(d == '\n') return 1;
+		return c < d ? -1 : 1;
+	}
+}
+
+/*********************************************************************
+fdIntoMemory reads data from a file descriptor, until EOF is reached.
+It works even if we don't know the size beforehand.
+We can now use it to read /proc files, pipes, and stdin.
+This solves an outstanding issue, and it is needed for forthcoming
+functionality, such as edpager.
+inpart = 0: read the whole file
+inpart = 1: read the first part
+inpart = 2: read the next part
+*********************************************************************/
+#define FILEPARTSIZE 10000000
+
+int fdIntoMemory(int fd, char **data, int *len, bool inparts)
+{
+	int length, n, j;
 	const int blocksize = 8192;
 	char *chunk, *buf;
+	static char *leftover;
+	static int lolen; // leftover length
 
 	chunk = allocZeroString(blocksize);
-	buf = initString(&length);
+	if(inparts <= 1)
+		buf = initString(&length);
+	else
+		buf = leftover, length = lolen;
 
-	n = 0;
-	do {
+	while(true) {
 		if(length >= 0x7fffff00 - blocksize) {
 			nzFree(buf);
 			nzFree(chunk);
 			*data = emptyString;
 			*len = 0;
 			setError(MSG_BigFile);
-			return false;
+			return 0;
 		}
 		n = read(fd, chunk, blocksize);
 		if (n < 0) {
@@ -734,44 +751,67 @@ bool fdIntoMemory(int fd, char **data, int *len)
 			*data = emptyString;
 			*len = 0;
 			setError(MSG_NoRead, "file descriptor");
-			return false;
+			return 0;
 		}
 
-		if (n > 0)
-			stringAndBytes(&buf, &length, chunk, n);
-	} while (n != 0);
+		if (!n) break;
+		stringAndBytes(&buf, &length, chunk, n);
+		if(!inparts || length < FILEPARTSIZE) continue;
+// Can't read in parts if chars are 16 bit or 32 bit wide
+		if(inparts == 1 && byteOrderMark((uchar *) buf, length)) {
+			inparts = 0; // back to default read
+			continue;
+		}
+// we have to have some leftover
+		for(j = length - 2; j > 0 && buf[j] != '\n'; --j)  ;
+		if(!j) continue;
+		++j;
+		leftover = initString(&lolen);
+		stringAndBytes(&leftover, &lolen, buf + j, length - j);
+		nzFree(chunk);
+// do we need room for the extra \n\0 on the end for a piece of the file?
+// I don't think so.
+		*data = buf;
+		*len = j;
+// success, but more to read
+		debugPrint(4, "part");
+		return 2;
+	}
 
 	nzFree(chunk);
 	buf = reallocString(buf, length + 2);
 	*data = buf;
 	*len = length;
-	return true;
-}				/* fdIntoMemory */
+	return 1;
+}
 
-bool fileIntoMemory(const char *filename, char **data, int *len)
+int fileIntoMemory(const char *filename, char **data, int *len, bool inparts)
 {
-	int fh;
-	char ftype = fileTypeByName(filename, 0);
-	bool ret;
+	static int fh;
+	char ftype;
+	int ret;
+
+	if(inparts == 2) goto fh_set;
+	ftype = fileTypeByName(filename, 0);
 	if (ftype && ftype != 'f' && ftype != 'p') {
 		setError(MSG_RegularFile, filename);
-		return false;
+		return 0;
 	}
 	fh = open(filename, O_RDONLY | O_BINARY);
 	if (fh < 0) {
 		setError(MSG_NoOpen, filename);
-		return false;
+		return 0;
 	}
 
-	ret = fdIntoMemory(fh, data, len);
-	if (ret == false)
+fh_set:
+	ret = fdIntoMemory(fh, data, len, inparts);
+	if (ret == 0 && filename)
 		setError(MSG_NoRead2, filename);
-
-	close(fh);
+	if(ret <= 1) close(fh);
 	return ret;
-}				/* fileIntoMemory */
+}
 
-/* inverse of the above */
+// inverse of the above
 bool memoryOutToFile(const char *filename, const char *data, int len,
 /* specify the error messages */
 		     int msgcreate, int msgwrite)
@@ -789,25 +829,101 @@ bool memoryOutToFile(const char *filename, const char *data, int len,
 	}
 	close(fh);
 	return true;
-}				/* memoryOutToFile */
+}
 
 // portable function to truncate to 0
 void truncate0(const char *filename, int fh)
 {
-#ifndef DOSLIKE
-// unix is easy
 	if (fh < 0)
 		truncate(filename, 0l);
 	else
 		ftruncate(fh, 0l);
-#else
-	int fh2;
-	if (fh >= 0)
-		lseek(fh, 0L, 0);
-	fh2 = open(filename, O_WRONLY | O_TRUNC);
-	if (fh2 >= 0)		// it should be
-		close(fh2);
-#endif
+}
+
+long long bufferSizeW(const Window *w, bool browsing)
+{
+	int ln;
+	long long size = 0;
+	pst p;
+	if (!w)
+		return -1;
+	for (ln = 1; ln <= w->dol; ++ln) {
+		p = w->map[ln].text;
+		while (*p != '\n') {
+			if (*p == InternalCodeChar && browsing && w->browseMode) {
+				++p;
+				while (isdigitByte(*p)) ++p;
+				if (strchr("<>{}", *p)) ++size;
+				++p;
+				continue;
+			}
+			++p, ++size;
+		}
+		++size;
+	}			// loop over lines
+	if (w->nlMode)
+		--size;
+	return size;
+}
+
+long long bufferSize(int cx, bool browsing)
+{
+	const Window *w;
+	if (cx <= 0 || cx >= MAXSESSION || (w = sessionList[cx].lw) == 0) {
+		setError(MSG_SessionInactive, cx);
+		return -1;
+	}
+	return bufferSizeW(w, browsing);
+}
+
+// Unfold the buffer into one long, allocated string
+bool unfoldBufferW(const Window *w, bool cr, char **data, int *len)
+{
+	char *buf;
+	int l, ln;
+	long long size = bufferSizeW(w, false);
+	if (size < 0)
+		return false;
+	if (size >= 2000000000) {
+		setError(MSG_BigFile);
+		return false;
+	}
+	if (w->dirMode) {
+		setError(MSG_SessionDir, context);
+		return false;
+	}
+	if (cr)
+		size += w->dol;
+/* a few bytes more, just for safety */
+	buf = allocMem(size + 4);
+	*data = buf;
+	for (ln = 1; ln <= w->dol; ++ln) {
+		pst line = w->map[ln].text;
+		l = pstLength(line) - 1;
+		if (l) {
+			memcpy(buf, line, l);
+			buf += l;
+		}
+		if (cr) {
+			*buf++ = '\r';
+			if (l && buf[-2] == '\r')
+				--buf, --size;
+		}
+		*buf++ = '\n';
+	}			// loop over lines
+	if (w->dol && w->nlMode) {
+		if (cr)
+			--size;
+	}
+	*len = size;
+	(*data)[size] = 0;
+	return true;
+}
+
+bool unfoldBuffer(int cx, bool cr, char **data, int *len)
+{
+	const Window *w = sessionList[cx].lw;
+	return unfoldBufferW(w, cr, data, len);
 }
 
 /* shift string to upper, lower, or mixed case */
@@ -850,7 +966,7 @@ void caseShift(char *s, char action)
 		}
 		ws = true, mc = 0;
 	}			/* loop */
-}				/* caseShift */
+}
 
 // foo-bar has to become fooBar
 void camelCase(char *s)
@@ -875,7 +991,7 @@ is ported to other operating systems.
  * I think this will work on Windows, not sure.
  * But the link feature is Unix specific.
  * Remember the stat structure for other things. */
-struct stat this_stat;
+static struct stat this_stat;
 static bool this_waslink, this_brokenlink;
 
 char fileTypeByName(const char *name, int showlink)
@@ -955,7 +1071,7 @@ off_t fileSizeByName(const char *name)
 		return -1;
 	}
 	return buf.st_size;
-}				/* fileSizeByName */
+}
 
 off_t fileSizeByHandle(int fd)
 {
@@ -965,7 +1081,7 @@ off_t fileSizeByHandle(int fd)
 		return -1;
 	}
 	return buf.st_size;
-}				/* fileSizeByHandle */
+}
 
 time_t fileTimeByName(const char *name)
 {
@@ -975,7 +1091,7 @@ time_t fileTimeByName(const char *name)
 		return -1;
 	}
 	return buf.st_mtime;
-}				/* fileTimeByName */
+}
 
 char *conciseSize(size_t n)
 {
@@ -1007,7 +1123,7 @@ char *conciseSize(size_t n)
 		sprintf(buf, "%lu", u);
 	}
 	return buf;
-}				/* conciseSize */
+}
 
 char *conciseTime(time_t t)
 {
@@ -1048,7 +1164,7 @@ char *conciseTime(time_t t)
 		allMonths[eb_lang][tm->tm_mon], tm->tm_mday, tm->tm_year + 1900,
 		tm->tm_hour, tm->tm_min);
 	return buffer;
-}				/* conciseTime */
+}
 
 /* retain only characters l s t i k p m, for ls attributes */
 bool lsattrChars(const char *buf, char *dest)
@@ -1056,11 +1172,7 @@ bool lsattrChars(const char *buf, char *dest)
 	bool rc = true;
 	const char *s;
 	char c, *t;
-#ifdef DOSLIKE
-	static const char ok_chars[] = "lst";
-#else
 	static const char ok_chars[] = "lstikpmy";
-#endif
 	char used[26];
 	memset(used, 0, sizeof(used));
 	t = dest;
@@ -1078,7 +1190,7 @@ bool lsattrChars(const char *buf, char *dest)
 	}
 	*t = 0;
 	return rc;
-}				/* lsattrChars */
+}
 
 // expand the ls attributes for a file into a static string.
 // This assumes user/group names will not be too long.
@@ -1113,17 +1225,12 @@ char *lsattr(const char *path, const char *flags)
 			break;
 		case 'l':
 			sprintf(p, "%lld", (long long)this_stat.st_size);
-#ifndef DOSLIKE
 p:
-#endif // #ifndef DOSLIKE
 			strcat(buf, p);
 			break;
 		case 's':
 			strcat(buf, conciseSize(this_stat.st_size));
 			break;
-#ifndef DOSLIKE
-/* not sure any of these work under windows */
-
 		case 'i':
 			sprintf(p, "%lu", (unsigned long)this_stat.st_ino);
 			goto p;
@@ -1189,9 +1296,6 @@ p:
 				s[l] = 0;
 			}
 			break;
-
-#endif
-
 		}
 
 		++flags;
@@ -1200,13 +1304,6 @@ p:
 	return buf;
 }
 
-#ifdef DOSLIKE
-void ttySaveSettings(void)
-{
-	// TODO: Anything needed here for WIN32?
-	isInteractive = _isatty(0);
-}
-#else // !#ifdef DOSLIKE
 static struct termios savettybuf;
 void ttySaveSettings(void)
 {
@@ -1240,11 +1337,9 @@ void ttyRaw(int charcount, int timeout, bool isecho)
 		buf.c_lflag |= ECHO;
 	tcsetattr(0, TCSANOW, &buf);
 }
-#endif // #ifdef DOSLIKE y/n
 
 void ttySetEcho(bool enable_echo)
 {
-#ifndef DOSLIKE
 	struct termios termios;
 
 	if (!isInteractive)
@@ -1259,10 +1354,8 @@ void ttySetEcho(bool enable_echo)
 		termios.c_lflag |= ECHONL;
 	}
 	tcsetattr(0, TCSANOW, &termios);
-#endif // #ifndef DOSLIKE
 }
 
-#ifndef DOSLIKE
 /* simulate MSDOS getche() system call */
 int getche(void)
 {
@@ -1284,7 +1377,6 @@ int getch(void)
 	return c;
 }
 
-#endif // #ifndef DOSLIKE
 
 char getLetter(const char *s)
 {
@@ -1350,7 +1442,7 @@ char *getFileName(int msg, const char *defname, bool isnew, bool ws)
 }
 
 /* Protect a filename from expansion by the shell */
-static const char shellmeta[] = "\\\n\t |&;<>(){}#'\"~$*?";
+static const char shellmeta[] = "\\\n\t |&;<>(){}#'\"~$*?`";
 int shellProtectLength(const char *s)
 {
 	int l = 0;
@@ -1360,7 +1452,7 @@ int shellProtectLength(const char *s)
 		++l, ++s;
 	}
 	return l;
-}				/* shellProtectLength */
+}
 
 void shellProtect(char *t, const char *s)
 {
@@ -1369,7 +1461,66 @@ void shellProtect(char *t, const char *s)
 			*t++ = '\\';
 		*t++ = *s++;
 	}
-}				/* shellProtect */
+}
+
+// get the directory suffix for a file.
+// This only makes sense in directory mode.
+char *dirSuffixContext(int n, int cx)
+{
+	static char suffix[4];
+	Window *lw = sessionList[cx].lw;
+
+	suffix[0] = 0;
+	if (lw->dirMode) {
+		suffix[0] = lw->dmap[DTSIZE*n];
+		suffix[1] = lw->dmap[DTSIZE*n + 1];
+		suffix[2] = 0;
+	}
+	return suffix;
+}
+
+char *dirSuffix(int n)
+{
+	return dirSuffixContext(n, context);
+}
+
+char *dirSuffix2(int n, const char *path)
+{
+	static char suffix[4], *t;
+	char ftype, c;
+	if(!cw->dnoMode)
+		return dirSuffixContext(n, context);
+// names only, don't have file type information, have to go get it
+	t = suffix;
+	ftype = fileTypeByName(path, 1);
+	if(isupper(ftype)) *t++ = '@', ftype = tolower(ftype);
+	c = 0;
+	if (ftype == 'd') c = '/';
+	if (ftype == 's') c = '^';
+	if (ftype == 'c') c = '<';
+	if (ftype == 'b') c = '*';
+	if (ftype == 'p') c = '|';
+	*t++ = c;
+	*t = 0;
+	return suffix;
+}
+
+// create the full pathname for a file that you are viewing in directory mode.
+// The return is static, with a limit on path length.
+char *makeAbsPath(const char *f)
+{
+	static char path[ABSPATH + 200];
+	const char *b = cw->baseDirName ? cw->baseDirName : emptyString;
+	if (strlen(b) + strlen(f) > ABSPATH - 2) {
+		setError(MSG_PathNameLong, ABSPATH);
+		return 0;
+	}
+	if(cw->baseDirName)
+		sprintf(path, "%s/%s", b, f);
+	else
+		strcpy(path, f);
+	return path;
+}
 
 /* loop through the files in a directory */
 const char *nextScanFile(const char *base)
@@ -1403,20 +1554,20 @@ const char *nextScanFile(const char *base)
 	closedir(df);
 	df = 0;
 	return 0;
-}				/* nextScanFile */
+}
 
-/* compare routine for quicksort directory scan */
+// compare routine for quicksort directory scan, alphabetical
 static bool dir_reverse;
-static int dircmp(const void *s, const void *t)
+static int dircmp_alph(const void *s, const void *t)
 {
 	int rc = strcoll((const char *)((const struct lineMap *)s)->text,
 			 (const char *)((const struct lineMap *)t)->text);
 	if (dir_reverse)
 		rc = -rc;
 	return rc;
-}				/* dircmp */
+}
 
-bool sortedDirList(const char *dir, struct lineMap ** map_p, int *count_p,
+static bool sortedDirList(const char *dir, struct lineMap ** map_p, int *count_p,
 		   int othersort, bool reverse)
 {
 	const char *f;
@@ -1435,7 +1586,6 @@ bool sortedDirList(const char *dir, struct lineMap ** map_p, int *count_p,
 /* leave room for @ / newline */
 		t->text = (pst) allocMem(strlen(f) + 3);
 		strcpy((char *)t->text, f);
-		t->ds1 = t->ds2 = 0;
 		++t, ++linecount;
 	}
 
@@ -1449,7 +1599,508 @@ bool sortedDirList(const char *dir, struct lineMap ** map_p, int *count_p,
 // unless we plan to sort them some other way.
 	if (!othersort) {
 		dir_reverse = reverse;
-		qsort(map, linecount, LMSIZE, dircmp);
+		qsort(map, linecount, LMSIZE, dircmp_alph);
+	}
+
+	return true;
+}
+
+// directory sort record, for nonalphabetical sorts.
+struct DSR {
+	int idx;
+	union {
+#ifdef linux
+		struct timespec spec;
+#else
+		time_t t;
+#endif
+		off_t z;
+	} u;
+};
+static struct DSR *dsr_list;
+uchar ls_sort;		// sort method for directory listing
+bool ls_reverse;		// reverse sort
+char lsformat[12];	// size date etc on a directory listing
+
+// compare routine for quicksort directory scan, size or time
+static int dircmp_st(const void *s, const void *t)
+{
+	const struct DSR *q = s;
+	const struct DSR *r = t;
+	int rc;
+	if (ls_sort == 1) {
+		rc = 0;
+		if (q->u.z < r->u.z)
+			rc = -1;
+		if (q->u.z > r->u.z)
+			rc = 1;
+	}
+	if (ls_sort == 2) {
+		rc = 0;
+#ifdef linux
+		if (q->u.spec.tv_sec < r->u.spec.tv_sec)
+			rc = -1;
+		if (q->u.spec.tv_sec > r->u.spec.tv_sec)
+			rc = 1;
+		if(!rc) {
+// Honor sub-second timestamp precision if the operating system supports it.
+		if (q->u.spec.tv_nsec < r->u.spec.tv_nsec)
+			rc = -1;
+		if (q->u.spec.tv_nsec > r->u.spec.tv_nsec)
+			rc = 1;
+		}
+#else
+		if (q->u.t < r->u.t)
+			rc = -1;
+		if (q->u.t > r->u.t)
+			rc = 1;
+#endif
+	}
+	if (ls_reverse)
+		rc = -rc;
+	return rc;
+}
+
+// Read the contents of a directory into the current buffer
+bool readDirectory(const char *filename, int endline, char cmd, struct lineMap **map_p)
+{
+	int len, j, linecount;
+	char *v;
+	char *dmap = 0;
+	struct lineMap *mptr;
+	struct lineMap *backpiece = 0;
+	uchar innersort = (dno ? 0 : ls_sort);
+	bool innerrev = (dno ? false : ls_reverse);
+
+	cw->baseDirName = cloneString(filename);
+/* get rid of trailing slash */
+	len = strlen(cw->baseDirName);
+	if (len && cw->baseDirName[len - 1] == '/')
+		cw->baseDirName[len - 1] = 0;
+/* Understand that the empty string now means / */
+
+/* get the files, or fail if there is a problem */
+	if (!sortedDirList
+	    (filename, map_p, &linecount, innersort, innerrev))
+		return false;
+	if (!cw->dol) {
+		cw->dirMode = true;
+		cw->dnoMode = dno;
+		dmap = allocZeroMem(linecount * DTSIZE);
+		if(debugLevel >= 1)
+			i_puts(MSG_DirMode);
+		if (lsformat[0])
+			backpiece = allocZeroMem(LMSIZE * (linecount + 2));
+	}
+
+	if (!linecount) {	/* empty directory */
+		cw->dot = endline;
+		fileSize = 0;
+		free(*map_p);
+		*map_p = 0;
+		nzFree(backpiece);
+		nzFree(dmap);
+		goto success;
+	}
+
+	if (innersort)
+		dsr_list = allocZeroMem(sizeof(struct DSR) * linecount);
+
+/* change 0 to nl and count bytes */
+	fileSize = 0;
+	mptr = *map_p;
+	for (j = 0; j < linecount; ++j, ++mptr) {
+		char c, ftype;
+		pst t = mptr->text;
+		char *abspath = makeAbsPath((char *)t);
+
+// make sure this gets done.
+		if (backpiece)
+			backpiece[j + 1].text = (uchar *) emptyString;
+
+		while (*t) {
+			if (*t == '\n')
+				*t = '\t';
+			++t;
+		}
+		*t = '\n';
+		len = t - mptr->text;
+		fileSize += len + 1;
+		if (innersort)
+			dsr_list[j].idx = j;
+		if (!abspath)
+			continue;	/* should never happen */
+
+		ftype = fileTypeByName(abspath, 2);
+		if (!ftype)
+			continue;
+		if (isupperByte(ftype)) {	/* symbolic link */
+			if (!cw->dirMode)
+				*t = '@', *++t = '\n';
+			else
+				dmap[j*DTSIZE] = '@';
+			++fileSize;
+		}
+		ftype = tolower(ftype);
+		c = 0;
+		if (ftype == 'd') c = '/';
+		if (ftype == 's') c = '^';
+		if (ftype == 'c') c = '<';
+		if (ftype == 'b') c = '*';
+		if (ftype == 'p') c = '|';
+		if (c) {
+			if (!cw->dirMode)
+				*t = c, *++t = '\n';
+			else if (dmap[j*DTSIZE])
+				dmap[DTSIZE*j + 1] = c;
+			else
+				dmap[DTSIZE*j] = c;
+			++fileSize;
+		}
+// If sorting a different way, get the attribute.
+		if (innersort) {
+			if (innersort == 1)
+				dsr_list[j].u.z = this_stat.st_size;
+			if (innersort == 2) {
+// Honor sub-second timestamp precision if the operating system supports it.
+// Currently only linux - other systems?
+#ifdef linux
+				dsr_list[j].u.spec = this_stat.st_mtim;
+#else
+				dsr_list[j].u.t = this_stat.st_mtime;
+#endif
+			}
+		}
+
+/* extra stat entries on the line */
+		if (!lsformat[0] || dno)
+			continue;
+		v = lsattr(abspath, lsformat);
+		if (!*v)
+			continue;
+		len = strlen(v);
+		fileSize += len + 1;
+		if (cw->dirMode) {
+			backpiece[j + 1].text = (uchar *) cloneString(v);
+		} else {
+/* have to realloc at this point */
+			int l1 = t - mptr->text;
+			mptr->text = reallocMem(mptr->text, l1 + len + 3);
+			t = mptr->text + l1;
+			*t++ = ' ';
+			strcpy((char *)t, v);
+			t += len;
+			*t++ = '\n';
+			*t = 0;
+		}
+	}			// loop fixing files in the directory scan
+
+	if (innersort) {
+		struct lineMap *tmp;
+		char *dmap2;
+		qsort(dsr_list, linecount, sizeof(struct DSR), dircmp_st);
+// Now I have to remap everything.
+		tmp = allocMem(LMSIZE * linecount);
+		if(dmap) dmap2 = allocZeroMem(DTSIZE * linecount);
+		for (j = 0; j < linecount; ++j) {
+			tmp[j] = (*map_p)[dsr_list[j].idx];
+			if(!dmap) continue;
+			dmap2[DTSIZE*j] = dmap[DTSIZE*dsr_list[j].idx];
+			dmap2[DTSIZE*j + 1] = dmap[DTSIZE*dsr_list[j].idx + 1];
+		}
+		free(*map_p);
+		*map_p = tmp;
+		if(dmap) free(dmap), dmap = dmap2;
+		if (backpiece) {
+			tmp = allocMem(LMSIZE * (linecount + 2));
+			for (j = 0; j < linecount; ++j)
+				tmp[j + 1] = backpiece[dsr_list[j].idx + 1];
+			memset(tmp, 0, LMSIZE);
+			memset(tmp + linecount + 1, 0, LMSIZE);
+			free(backpiece);
+			backpiece = tmp;
+		}
+		free(dsr_list);
+	}
+
+	addToMap(linecount, endline);
+	cw->r_map = backpiece;
+	if(dmap) {
+		cw->dmap = allocMem((linecount + 1) * DTSIZE);
+		memcpy(cw->dmap + DTSIZE, dmap, linecount*DTSIZE);
+	}
+
+success:
+	if (cmd == 'r')
+		debugPrint(1, "%lld", fileSize);
+	return true;
+}
+
+// Delete files from a directory as you delete lines.
+// Set dw to move them to your recycle bin.
+// Set dx to delete them outright.
+bool delFiles(int start, int end, bool withtext, char origcmd, char *cmd_p)
+{
+	int ln, j;
+
+	if (!dirWrite) {
+		setError(MSG_DirNoWrite);
+		return false;
+	}
+
+	if (dirWrite == 1 && !recycleBin) {
+		setError(MSG_NoRecycle);
+		return false;
+	}
+
+	if(end < start) return true;
+	*cmd_p = 'e';		// show errors
+
+	for (ln = start; ln <= end; ++ln) {
+		char *file, *t, *path, *ftype, *a;
+		char qc = '\''; // quote character
+		file = (char *)fetchLine(ln, 0);
+		t = strchr(file, '\n');
+		if (!t)
+			i_printfExit(MSG_NoNlOnDir, file);
+		*t = 0;
+		path = makeAbsPath(file);
+		if (!path) {
+abort:
+			free(file);
+			if (ln != start && withtext)
+				delText(start, ln - 1);
+			return false;
+		}
+
+// check formeta chars in path
+		if(strchr(path, qc)) {
+			qc = '"';
+			if(strpbrk(path, "\"$"))
+// I can't easily turn this into a shell command, so just hang it.
+				qc = 0;
+		}
+		if(strstr(path, "\\\\"))
+			qc = 0;
+
+		ftype = dirSuffix2(ln, path);
+		if (dirWrite == 2 || (*ftype && strchr("@<*^|", *ftype)))
+			debugPrint(1, "%s%s ↓", file, ftype);
+		else
+			debugPrint(1, "%s%s → 🗑", file, ftype);
+
+		if (dirWrite == 2 && *ftype == '/') {
+			if(!qc) {
+				setError(MSG_MetaChar);
+				goto abort;
+			}
+			asprintf(&a, "rm -rf %c%s%c",
+			qc, path, qc);
+			j = system(a);
+			free(a);
+			if(!j) {
+				free(file);
+				continue;
+			} else {
+				setError(MSG_NoDirDelete);
+				goto abort;
+			}
+		}
+
+		if (dirWrite == 2 || (*ftype && strchr("@<*^|", *ftype))) {
+unlink:
+			if (unlink(path)) {
+				setError(MSG_NoRemove, file);
+				goto abort;
+			}
+		} else {
+			char bin[ABSPATH];
+			sprintf(bin, "%s/%s", recycleBin, file);
+			if (rename(path, bin)) {
+				if (errno == EXDEV) {
+					char *rmbuf;
+					int rmlen;
+					if (*ftype == '/' ||
+					fileSizeByName(path) > 200000000) {
+// let mv do the work
+						if(!qc) {
+							setError(MSG_MetaChar);
+							goto abort;
+						}
+						asprintf(&a, "mv -n %c%s%c",
+						qc, path, qc);
+						j = system(a);
+						free(a);
+						if(!j) {
+							free(file);
+							continue;
+						} else {
+							setError(MSG_MoveFileSystem , path);
+							goto abort;
+						}
+					}
+					if (!fileIntoMemory    (path, &rmbuf, &rmlen, 0))
+						goto abort;
+					if (!memoryOutToFile(bin, rmbuf, rmlen,
+							     MSG_TempNoCreate2,
+							     MSG_NoWrite2)) {
+						nzFree(rmbuf);
+						goto abort;
+					}
+					nzFree(rmbuf);
+					goto unlink;
+				}
+
+// some other rename error
+				setError(MSG_NoMoveToTrash, file);
+				goto abort;
+			}
+		}
+	}
+	if(withtext) delText(start, end);
+
+// if you type D instead of d, I don't want to lose that.
+	*cmd_p = origcmd;
+	return true;
+}
+
+// Move or copy files from one directory to another
+bool moveFiles(int start, int end, int dest, char origcmd)
+{
+	Window *cw1 = cw;
+	Window *cw2 = sessionList[dest].lw;
+	char *path1, *path2;
+	int ln, cnt, dol;
+
+	if (!dirWrite) {
+		setError(MSG_DirNoWrite);
+		return false;
+	}
+
+	ln = start;
+	cnt = end - start + 1;
+	while (cnt--) {
+		char *file, *t, *ftype;
+		file = (char *)fetchLine(ln, 0);
+		t = strchr(file, '\n');
+		if (!t)
+			i_printfExit(MSG_NoNlOnDir, file);
+		*t = 0;
+		path1 = makeAbsPath(file);
+		if (!path1) {
+			free(file);
+			return false;
+		}
+		path1 = cloneString(path1);
+		ftype = dirSuffix2(ln, path1);
+
+		debugPrint(1, "%s%s %s %s",
+		file, ftype, (origcmd == 'm' ? "→" : "≡"), cw2->baseDirName);
+
+
+		cw = cw2;
+		path2 = makeAbsPath(file);
+		cw = cw1;
+		if (!path2) {
+			free(file);
+			free(path1);
+			return false;
+		}
+
+		if (!access(path2, 0)) {
+			setError(MSG_DestFileExists);
+			free(file);
+			free(path1);
+			return false;
+		}
+
+		errno = EXDEV;
+		if (origcmd == 't' || rename(path1, path2)) {
+			if (errno == EXDEV) {
+				char *rmbuf;
+				int rmlen, j;
+				if (*ftype || fileSizeByName(path1) > 200000000) {
+// let mv or cp do the work
+					char *a, qc = '\'';
+					if(strchr(path1, qc) || strchr(path2, qc)) {
+						qc = '"';
+						if(strpbrk(path1, "\"$") || strpbrk(path2, "\"$"))
+// I can't easily turn this into a shell command, so just hang it.
+							qc = 0;
+					}
+					if(strstr(path1, "\\\\") || strstr(path2, "\\\\"))
+						qc = 0;
+					if(!qc) {
+						setError(MSG_MetaChar);
+						free(file);
+						free(path1);
+						return false;
+					}
+					asprintf(&a, "%s %c%s%c %c%s%c",
+					(origcmd == 'm' ? "mv -n" : "cp -an"),
+					qc, path1, qc, qc, cw2->baseDirName, qc);
+					j = system(a);
+					free(a);
+					if(j) {
+						setError((origcmd == 'm' ? MSG_MoveFileSystem : MSG_CopyFail), file);
+						free(file);
+						free(path1);
+						return false;
+					}
+					if(origcmd == 'm')
+						unlink(path1);
+					goto moved;
+				}
+// A small file, copy it ourselves.
+				if (!fileIntoMemory(path1, &rmbuf, &rmlen, 0)) {
+					free(file);
+					free(path1);
+					return false;
+				}
+				if (!memoryOutToFile(path2, rmbuf, rmlen,
+						     MSG_TempNoCreate2,
+						     MSG_NoWrite2)) {
+					free(file);
+					free(path1);
+					nzFree(rmbuf);
+					return false;
+				}
+				nzFree(rmbuf);
+				if(origcmd == 'm')
+					unlink(path1);
+				goto moved;
+			}
+
+			setError(MSG_MoveError, file, errno);
+			free(file);
+			free(path1);
+			return false;
+		}
+
+moved:
+		free(path1);
+		if(origcmd == 'm')
+			delText(ln, ln);
+// add it to the other directory
+		*t++ = '\n';
+		cw = cw2;
+		dol = cw->dol;
+		addTextToBuffer((pst)file, t-file, dol, false);
+		free(file);
+		cw->dot = ++dol;
+		cw->dmap = reallocMem(cw->dmap, DTSIZE * (dol + 1));
+		memset(cw->dmap + DTSIZE*dol, 0, DTSIZE);
+		cw->dmap[DTSIZE*dol] = ftype[0];
+		if(ftype[0])
+		cw->dmap[DTSIZE*dol + 1] = ftype[1];
+// if attributes were displayed in that directory - more work to do.
+// I just leave a space for them; I don't try to derive them.
+		if(cw->r_map) {
+			cw->r_map = reallocMem(cw->r_map, LMSIZE * (dol + 2));
+			memset(cw->r_map + dol, 0, LMSIZE*2);
+			cw->r_map[dol].text = (uchar*)emptyString;
+		}
+		cw = cw1; // put it back
 	}
 
 	return true;
@@ -1498,7 +2149,6 @@ static bool envExpand(const char *line, const char **expanded)
 	udir = 0;
 	strncpy(var1, line + 1, l);
 	var1[l] = 0;
-#ifndef DOSLIKE
 	if (l) {
 		pw = getpwnam(var1);
 		if (!pw) {
@@ -1508,7 +2158,6 @@ static bool envExpand(const char *line, const char **expanded)
 		if (pw->pw_dir && *pw->pw_dir)
 			udir = pw->pw_dir;
 	} else
-#endif
 		udir = home;
 	if (!udir) {
 		s = line;
@@ -1572,7 +2221,7 @@ appendchar:
 
 	*expanded = varline;
 	return true;
-}				/* envExpand */
+}
 
 bool envFile(const char *line, const char **expanded)
 {
@@ -1580,9 +2229,7 @@ bool envFile(const char *line, const char **expanded)
 	const char *varline;
 	const char *s;
 	char *t;
-#ifndef DOSLIKE
 	glob_t g;
-#endif // #ifndef DOSLIKE
 	int rc, flags;
 
 /* ` disables this stuff */
@@ -1598,11 +2245,6 @@ bool envFile(const char *line, const char **expanded)
 	if (!envExpand(line, &varline))
 		return false;
 
-#ifdef DOSLIKE
-	*expanded = varline;
-	return true;		// TODO: WIN32: Expand like glob...
-#else // !#ifdef DOSLIKE
-
 /* expanded the environment variables, if any, now time to glob */
 	flags = GLOB_NOSORT;
 	rc = glob(varline, flags, NULL, &g);
@@ -1645,8 +2287,6 @@ bool envFile(const char *line, const char **expanded)
 	globfree(&g);
 	*expanded = line2;
 	return true;
-#endif // #ifdef DOSLIKE y/n
-
 }
 
 // Like the above, but string is allocated, and errors are printed
